@@ -8,7 +8,7 @@
  * Copyright (C) 1999-2003.  The University of Chicago
  * See the file LICENSE for information on usage and redistribution.
  *
- * $Header: /cvsroot/swig/SWIG/Source/CParse/cparse.h,v 1.11 2004/12/01 01:01:01 marcelomatus Exp $
+ * $Header: /cvsroot/SWIG/Source/CParse/cparse.h,v 1.4 2003/12/28 21:44:41 cheetah Exp $
  * ----------------------------------------------------------------------------- */
 
 #ifndef CPARSE_H_
@@ -43,24 +43,16 @@ extern int       yylex();
 /* parser.y */
 extern SwigType *Swig_cparse_type(String *);
 extern Node     *Swig_cparse(File *);
-extern Hash     *Swig_cparse_features();
-extern void      SWIG_cparse_set_compact_default_args(int defargs);
-extern void      SWIG_cparse_template_reduce(int treduce);
 
 /* util.c */
 extern void      Swig_cparse_replace_descriptor(String *s);
 extern void      cparse_normalize_void(Node *);
 extern int       need_protected(Node *n, int dirprot_mode);
 extern Parm     *Swig_cparse_parm(String *s);
-extern int       need_name_warning(Node *n);
-extern int       need_redefined_warn(Node* a, Node* b, int InClass);
-
 
 /* templ.c */
-extern void     Swig_cparse_template_defargs(Parm *parms, Parm *targs, Symtab *tscope);
-extern String*  Swig_cparse_template_deftype(SwigType *type, Symtab *tscope);
-extern int       Swig_cparse_template_expand(Node *n, String *rname, ParmList *tparms, Symtab *tscope);
-extern Node     *Swig_cparse_template_locate(String *name, ParmList *tparms, Symtab *tscope);
+extern int       Swig_cparse_template_expand(Node *n, String *rname, ParmList *tparms);
+extern Node     *Swig_cparse_template_locate(String *name, ParmList *tparms);
 extern void      Swig_cparse_debug_templates(int);
 
 #ifdef __cplusplus
