@@ -10,7 +10,7 @@
  * See the file LICENSE for information on usage and redistribution.	
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_memory_c[] = "$Header: /cvsroot/swig/SWIG/Source/DOH/memory.c,v 1.6 2004/09/26 01:14:49 marcelomatus Exp $";
+char cvsroot_memory_c[] = "$Header: /cvsroot/SWIG/Source/DOH/memory.c,v 1.4 2003/06/25 09:36:59 cheetah Exp $";
 
 #include "dohint.h"
 
@@ -27,8 +27,6 @@ typedef struct pool {
   int            len;            /* Length of pool */
   int            blen;           /* Byte length of pool */
   int            current;        /* Current position for next allocation */
-  char*          pbeg;           /* Beg of pool */
-  char*          pend;           /* End of pool */
   struct  pool  *next;           /* Next pool */
 } Pool;
 
@@ -51,8 +49,6 @@ CreatePool() {
   p->len = PoolSize;
   p->blen = PoolSize*sizeof(DohBase);
   p->current = 0;
-  p->pbeg = ((char *)p->ptr);
-  p->pend = p->pbeg + p->blen;
   p->next = Pools;
   Pools = p;
 }
@@ -78,10 +74,12 @@ InitPools() {
 
 int 
 DohCheck(const DOH *ptr) {
-  register Pool *p = Pools;
+  Pool *p = Pools;
   register char *cptr = (char *) ptr;
+  register char *pptr;
   while (p) {
-    if ((cptr >= p->pbeg) && (cptr < p->pend)) return 1;
+    pptr = (char *) p->ptr;
+    if ((cptr >= pptr) && (cptr < (pptr + p->blen))) return 1;
     /*
     pptr = (char *) p->ptr;
     if ((cptr >= pptr) && (cptr < (pptr+(p->current*sizeof(DohBase))))) return 1; */
@@ -141,7 +139,7 @@ DohObjFree(DOH *ptr) {
   DohBase  *b, *meta;
   b = (DohBase *) ptr;
   if (b->flag_intern) return;
-  meta = (DohBase *)b->meta;
+  meta = b->meta;
   b->data = (void *) FreeList;
   b->meta = 0;
   b->type = 0;
@@ -189,7 +187,7 @@ DohMemoryDebug(void) {
 	else if (p->ptr[i].type == &DohHashType) numhash++;
       }
     }
-    printf("    Pool %8p: size = %10d. used = %10d. free = %10d\n", (void *)p, p->len, nused, nfree);
+    printf("    Pool %8p: size = %10d. used = %10d. free = %10d\n", p, p->len, nused, nfree);
     totsize += p->len;
     totused+= nused;
     totfree+= nfree;
