@@ -14,7 +14,7 @@
 
 #include "swigmod.h"
 
-char cvsroot_emit_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/emit.cxx,v 1.36 2004/10/20 20:59:33 wsfulton Exp $";
+char cvsroot_emit_cxx[] = "$Header: /cvsroot/SWIG/Source/Modules/emit.cxx,v 1.24 2003/12/28 21:47:58 cheetah Exp $";
 
 /* -----------------------------------------------------------------------------
  * emit_args()
@@ -36,13 +36,17 @@ void emit_args(SwigType *rt, ParmList *l, Wrapper *f) {
 
   /* Handle return type */
   if (rt && (SwigType_type(rt) != T_VOID)) {
-    SwigType *vt = 0;
-    vt = cplus_value_type(rt);
-    if (!vt) {
+    if (!CPlusPlus || (CPlusPlus && !SwigType_isclass(rt))) {
       Wrapper_add_local(f,"result", SwigType_lstr(rt,"result"));
     } else {
-      Wrapper_add_local(f,"result", SwigType_lstr(vt,"result"));
-      Delete(vt);
+      SwigType *vt = 0;
+      vt = cplus_value_type(rt);
+      if (!vt) {
+	Wrapper_add_local(f,"result", SwigType_lstr(rt,"result"));
+      } else {
+	Wrapper_add_local(f,"result", SwigType_lstr(vt,"result"));
+	Delete(vt);
+      }
     }
   }
   
@@ -184,7 +188,7 @@ void emit_attach_parmmaps(ParmList *l, Wrapper *f) {
 }
 
 /* -----------------------------------------------------------------------------
- * emit_num_arguments()
+ * emit_num_arguments()                                         ** new in 1.3.10
  *
  * Calculate the total number of arguments.   This function is safe for use
  * with multi-valued typemaps which may change the number of arguments in
@@ -216,30 +220,23 @@ int emit_num_arguments(ParmList *parms) {
 }
 
 /* -----------------------------------------------------------------------------
- * emit_num_required()
+ * emit_num_required()                                          ** new in 1.3.10
  *
- * Computes the number of required arguments.  This function is safe for
+ * Computes the number of required arguments.  This is function is safe for
  * use with multi-valued typemaps and knows how to skip over everything
- * properly. Note that parameters with default values are counted unless
- * the compact default args option is on.
+ * properly.
  * ----------------------------------------------------------------------------- */
 
 int emit_num_required(ParmList *parms) {
   Parm *p = parms;
   int   nargs = 0;
-  Parm *first_default_arg = 0;
-  int   compactdefargs = ParmList_is_compactdefargs(p);
 
   while (p) {
     if (Getattr(p,"tmap:in") && checkAttribute(p,"tmap:in:numinputs","0")) {
       p = Getattr(p,"tmap:in:next");
     } else {
+      if (Getattr(p,"value")) break;
       if (Getattr(p,"tmap:default")) break;
-      if (Getattr(p,"value")) {
-        if (!first_default_arg)
-          first_default_arg = p;
-        if (compactdefargs) break;
-      }
       nargs+= GetInt(p,"tmap:in:numinputs");
       if (Getattr(p,"tmap:in")) {
 	p = Getattr(p,"tmap:in:next");
@@ -249,26 +246,21 @@ int emit_num_required(ParmList *parms) {
     }
   }
 
-  /* Print error message for non-default arguments following default arguments */
-  /* The error message is printed more than once with most language modules, this ought to be fixed */
-  if (first_default_arg) {
-    p = first_default_arg;
-    while (p) {
-      if (Getattr(p,"tmap:in") && checkAttribute(p,"tmap:in:numinputs","0")) {
-        p = Getattr(p,"tmap:in:next");
+  /* Print message for non-default arguments */
+  while (p) {
+    if (Getattr(p,"tmap:in") && checkAttribute(p,"tmap:in:numinputs","0")) {
+      p = Getattr(p,"tmap:in:next");
+    } else {
+      if (!Getattr(p,"value") && (!Getattr(p,"tmap:default"))) {
+	Swig_error(Getfile(p),Getline(p),"Error. Non-optional argument '%s' follows an optional argument.\n",Getattr(p,"name"));
+      }
+      if (Getattr(p,"tmap:in")) {
+	p = Getattr(p,"tmap:in:next");
       } else {
-        if (!Getattr(p,"value") && (!Getattr(p,"tmap:default"))) {
-          Swig_error(Getfile(p),Getline(p),"Non-optional argument '%s' follows an optional argument.\n",Getattr(p,"name"));
-        }
-        if (Getattr(p,"tmap:in")) {
-          p = Getattr(p,"tmap:in:next");
-        } else {
-          p = nextSibling(p);
-        }
+	p = nextSibling(p);
       }
     }
   }
-
   /* DB 04/02/2003: Not sure this is necessary with tmap:in:numinputs */
   /*
   if (parms && (p = Getattr(parms,"emit:varargs"))) {
@@ -372,7 +364,7 @@ void emit_action(Node *n, Wrapper *f) {
     action = Getattr(n,"wrap:action");
   assert(action != 0);
 
-  if (!(is_public(n)) && is_member_director(n)) {
+  if (is_protected(n) && is_member_director(n)) {
     /* We need to add an extra dynamic_cast to
        access the director class, where the virtual
        methods are all public */
@@ -383,15 +375,6 @@ void emit_action(Node *n, Wrapper *f) {
     Wrapper_add_local(f, "darg", dirdecl);
     Printf(f->code, "darg = dynamic_cast<%s *>(arg1);\n",dirname); 
     Replace(action, "arg1", "darg", DOH_REPLACE_FIRST);
-    if (Getattr(n,"qualifier")) {
-      /* fix constant casting introduced by a const method decl */
-      String* classtype = Getattr(parent, "classtype");
-      /*
-	String *ccast = NewStringf("((%s const *)darg)",classtype);
-	if (Strstr(action,ccast) != 0) 
-      */
-      Replace(action, classtype, dirname, DOH_REPLACE_FIRST);
-    }
     Delete(dirname);
     Delete(dirdecl);
   }
@@ -410,42 +393,11 @@ void emit_action(Node *n, Wrapper *f) {
   }
   /* Exception handling code */
 
-  /* saves action -> eaction for postcatching exception */
-  String *eaction = NewString("");
-  
-  /* If we are in C++ mode and there is an exception specification. We're going to
+  /* If we are in C++ mode and there is a throw specifier. We're going to
      enclose the block in a try block */
-  if (throws) {
-    Printf(eaction,"try {\n");
-  }
-
-  Printv(eaction, action, NIL);
 
   if (throws) {
-    int unknown_catch = 0;
-    Printf(eaction,"}\n");
-    for (Parm *ep = throws; ep; ep = nextSibling(ep)) {
-      String *em = Swig_typemap_lookup_new("throws",ep,"_e",0);
-      if (em) {
-        SwigType *et = Getattr(ep,"type");
-        SwigType *etr = SwigType_typedef_resolve_all(et);
-        if (SwigType_isreference(etr) || SwigType_ispointer(etr) || SwigType_isarray(etr)) {
-          Printf(eaction,"catch(%s) {\n", SwigType_str(et, "_e"));
-        } else {
-          Printf(eaction,"catch(%s) {\n", SwigType_str(et, "&_e"));
-        }
-	Printv(eaction,em,"\n",NIL);
-	Printf(eaction,"}\n");
-      } else {
-	Swig_warning(WARN_TYPEMAP_THROW, Getfile(n), Getline(n),
-		     "No 'throws' typemap defined for exception type '%s'\n", 
-		     SwigType_str(Getattr(ep,"type"),0));
-	unknown_catch = 1;
-      }
-    }
-    if (unknown_catch) {
-      Printf(eaction,"catch(...) { throw; }\n");
-    }
+    Printf(f->code,"try {\n");
   }
 
   /* Look for except typemap (Deprecated) */
@@ -455,18 +407,33 @@ void emit_action(Node *n, Wrapper *f) {
   if (!tm) {
     tm = Getattr(n,"feature:except");
     if (tm) tm = Copy(tm);
-  }  
+  }
   if ((tm) && Len(tm) && (Strcmp(tm,"1") != 0)) {
     Replaceall(tm,"$name",Getattr(n,"name"));
     Replaceall(tm,"$symname", Getattr(n,"sym:name"));
-    Replaceall(tm,"$function", eaction);
-    Replaceall(tm,"$action", eaction);
+    Replaceall(tm,"$function", action);
+    Replaceall(tm,"$action", action);
     Printv(f->code,tm,"\n", NIL);
     Delete(tm);
   } else {
-    Printv(f->code,eaction,"\n",NIL);
+    Printv(f->code, action, "\n",NIL);
+  }
+
+  if (throws) {
+    Printf(f->code,"}\n");
+    for (Parm *ep = throws; ep; ep = nextSibling(ep)) {
+      String *em = Swig_typemap_lookup_new("throws",ep,"_e",0);
+      if (em) {
+	Printf(f->code,"catch(%s) {\n", SwigType_str(Getattr(ep,"type"),"&_e"));
+	Printv(f->code,em,"\n",NIL);
+	Printf(f->code,"}\n");
+      } else {
+	Swig_warning(WARN_TYPEMAP_THROW, Getfile(n), Getline(n),
+		     "No 'throw' typemap defined for exception type '%s'\n", SwigType_str(Getattr(ep,"type"),0));
+      }
+    }
+    Printf(f->code,"catch(...) { throw; }\n");
   }
-  Delete(eaction);
 
   /* Emit contract code (if any) */
   if (Swig_contract_mode_get()) {
