@@ -13,10 +13,10 @@
  * can be used and distributed.
  *****************************************************************************/
 
-char cvsroot_guile_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/guile.cxx,v 1.25 2004/11/04 01:02:21 wuzzeb Exp $";
+char cvsroot_guile_cxx[] = "$Header: /cvsroot/SWIG/Source/Modules/guile.cxx,v 1.11 2003/11/21 09:18:10 mkoeppe Exp $";
 
 /***********************************************************************
- * $Header: /cvsroot/swig/SWIG/Source/Modules/guile.cxx,v 1.25 2004/11/04 01:02:21 wuzzeb Exp $
+ * $Header: /cvsroot/SWIG/Source/Modules/guile.cxx,v 1.11 2003/11/21 09:18:10 mkoeppe Exp $
  *
  * guile.cxx
  *
@@ -30,11 +30,15 @@ char cvsroot_guile_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/guile.cxx
 
 #include "swigmod.h"
 
+#ifndef MACSWIG
+#include "swigconfig.h"
+#endif
 #include <ctype.h>
 
 // Note string broken in half for compilers that can't handle long strings
 static const char *guile_usage = (char*)"\
 Guile Options (available with -guile)\n\
+     -ldflags                - Print runtime libraries to link with\n\
      -prefix <name>          - Use <name> as prefix [default \"gswig_\"]\n\
      -package <name>         - Set the path of the module to <name>\n\
                                (default NULL)\n\
@@ -49,15 +53,15 @@ Guile Options (available with -guile)\n\
      -linkage <lstyle>       - Use linkage protocol <lstyle> (default `simple')\n\
                                Use `module' for native Guile module linking\n\
                                (requires Guile >= 1.5.0).  Use `passive' for\n\
-                               passive linking (no C-level module-handling code),\n\
+                               passive linking (no C-level module-handling  code),\n\
                                `ltdlmod' for Guile's old dynamic module\n\
                                convention (Guile <= 1.4), or `hobbit' for hobbit\n\
                                modules.\n\
      -scmstub                - Output Scheme file with module declaration and\n\
                                exports; only with `passive' and `simple' linkage\n\
-     -gh                     - Use the gh_ Guile API. (Guile <= 1.8) \n\
-     -scm                    - Use the scm Guile API. (Guile >= 1.6, default) \n\
-     -proxy                  - Export GOOPS class definitions\n\
+     -gh                     - Use the gh_ Guile API. (Guile <= 1.8, default) \n\
+     -scm                    - Use the scm Guile API.  (Guile >= 1.6) \n\
+     -shadow                 - Export GOOPS class definitions\n\
      -emitslotaccessors      - Emit accessor methods for all GOOPS slots\n" "\
      -primsuffix <suffix>    - Name appended to primitive module when exporting\n\
                                GOOPS classes. (default = \"primitive\")\n\
@@ -109,12 +113,13 @@ static String *return_multi_doc = 0;
 
 static String *exported_symbols = 0;
 
-static int     use_scm_interface = 1;
+static int     use_scm_interface = 0;
 static int     exporting_destructor = 0;
 static String *swigtype_ptr = 0;
 
 /* GOOPS stuff */
 static String *primsuffix = 0;
+static Hash   *known_classes = 0;
 static String *class_name = 0;
 static String *short_class_name = 0;
 static String *goops_class_methods;
@@ -168,6 +173,10 @@ public:
 	    Swig_arg_error();
 	  }
 	}
+	else if (strcmp (argv[i], "-ldflags") == 0) {
+	  printf("%s\n", use_scm_interface ? SWIG_GUILESCM_RUNTIME : SWIG_GUILE_RUNTIME);
+	  SWIG_exit (EXIT_SUCCESS);
+	}
 	else if (strcmp (argv[i], "-Linkage") == 0
 		 || strcmp (argv[i], "-linkage") == 0) {
 	  if (argv[i + 1]) {
@@ -232,7 +241,7 @@ public:
           scmstub = true;
           Swig_mark_arg(i);
 	}
-	else if ((strcmp(argv[i],"-shadow") == 0) || ((strcmp(argv[i],"-proxy") == 0))) {
+        else if (strcmp (argv[i], "-shadow") == 0) {
           goops = true;
           Swig_mark_arg(i);
         }
@@ -289,7 +298,7 @@ public:
     }
 
     if (goops) {
-      // -proxy implies -emit-setters
+      // -shadow implies -emit-setters
       emit_setters = 1;
     }
 
@@ -351,21 +360,19 @@ public:
     Swig_register_filebyname("goops", goopstext);
     goopscode = NewString("");
     goopsexport = NewString("");
+    known_classes = NewHash();
     
     Printf(f_runtime, "/* -*- buffer-read-only: t -*- vi: set ro: */\n");
     Swig_banner (f_runtime);
     
     Printf (f_runtime, "/* Implementation : GUILE */\n\n");
-
-    if (!use_scm_interface) {
-      if (SwigRuntime == 1)
-	Printf(f_runtime, "#define SWIG_GLOBAL\n");
-      if (SwigRuntime == 2)
-        Printf(f_runtime, "#define SWIG_NOINCLUDE\n");
-    }
     
     /* Write out directives and declarations */
     
+    if (NoInclude) {
+      Printf(f_runtime, "#define SWIG_NOINCLUDE\n");
+    }
+    
     module = Swig_copy_string(Char(Getattr(n,"name")));
     
     if (CPlusPlus) {
@@ -421,6 +428,7 @@ public:
     Delete(goopscode);
     Delete(goopsexport);
     Delete(goopstext);
+    Delete(known_classes);
 
     /* Close all of the files */
     Dump(f_header,f_runtime);
@@ -650,8 +658,7 @@ public:
 				    const String *maybe_delimiter,
 				    Parm *p,
 				    const String *typemap,
-				    const String *default_doc,
-				    const String *name = NULL)
+				    const String *default_doc)
   {
     String *tmp = NewString("");
     String *tm;
@@ -663,7 +670,7 @@ public:
     if (maybe_delimiter && Len(output) > 0 && Len(tm) > 0) {
       Printv(output, maybe_delimiter, NIL);
     }
-    const String *pn = (name == NULL) ? Getattr(p,"name") : name;
+    String *pn = Getattr(p,"name");
     String *pt = Getattr(p,"type");
     Replaceall(tm, "$name", pn); // legacy for $parmname
     Replaceall(tm, "$type", SwigType_str(pt,0));
@@ -699,7 +706,6 @@ public:
     String *returns = NewString("");
     String *method_signature = NewString("");
     String *primitive_args = NewString("");
-    Hash *scheme_arg_names = NewHash();
     int num_results = 1;
     String *tmp = NewString("");
     String *tm;
@@ -709,7 +715,6 @@ public:
     String *overname = 0;
     int args_passed_as_array = 0;
     int scheme_argnum = 0;
-    bool any_specialized_arg = false;
     
     // Make a wrapper name for this
     String *wname = Swig_name_wrapper(iname);
@@ -801,17 +806,6 @@ public:
 	Replaceall(tm,"$input",source);
 	Setattr(p,"emit:input", source);
 	Printv(f->code,tm,"\n",NIL);
-
-	SwigType *pb = SwigType_typedef_resolve_all(SwigType_base(pt));
-	SwigType *pn = Getattr(p,"name");
-	String *argname;
-	scheme_argnum++;
-	if (pn && !Getattr(scheme_arg_names, pn))
-	  argname = pn;
-	else {
-	  /* Anonymous arg or re-used argument name -- choose a name that cannot clash */
-	  argname = NewStringf("%%arg%d", scheme_argnum);
-	}
 	
 	if (procdoc) {
 	  if (i == numreq) {
@@ -820,34 +814,39 @@ public:
 	  }
 	  /* Add to signature (arglist) */
 	  handle_documentation_typemap(signature, " ", p, "tmap:in:arglist",
-				       "$name", argname);
+				       "$name");
 	  /* Document the type of the arg in the documentation body */
 	  handle_documentation_typemap(doc_body, ", ", p, "tmap:in:doc",
-				       "$NAME is of type <$type>", argname);
+				       "$NAME is of type <$type>");
 	}
 
 	if (goops) {
 	  if (i < numreq) {
+	    SwigType *pb = SwigType_typedef_resolve_all(SwigType_base(pt));
+	    SwigType *pn = Getattr(p,"name");
+	    String *argname;
+	    scheme_argnum++;
+	    if (pn)
+	      argname = pn;
+	    else {
+	      /* Anonymous arg -- choose a name that cannot clash */
+	      argname = NewStringf("%%arg%d", scheme_argnum);
+	    }
 	    if (strcmp("void", Char(pt)) != 0) {
-	      Node *class_node = Swig_symbol_clookup(pb, Getattr(n, "sym:symtab"));
-	      String *goopsclassname = (class_node == NULL) ? NULL :
-		Getattr(class_node, "guile:goopsclassname");
+	      Node *search;
 	      /* do input conversion */
-	      if (goopsclassname) {
-		Printv(method_signature, " (", argname, " ", goopsclassname, ")", NIL);
-		any_specialized_arg = true;
+	      if ((search = Getattr(known_classes, pb))) {
+		Printv(method_signature, " (", argname, " <", pb, ">)", NIL);
 	      } else {
 		Printv(method_signature, " ", argname, NIL);
 	      }
 	      Printv(primitive_args, " ", argname, NIL);
-	      Setattr(scheme_arg_names, argname, p);
+	    }
+	    if (!pn) {
+	      Delete(argname);
 	    }
 	  }
 	}
-	
-	if (!pn) {
-	  Delete(argname);
-	}
 	p = Getattr(p,"tmap:in:next");
       } else {
 	throw_unhandled_guile_type_error (pt);
@@ -1094,13 +1093,11 @@ public:
 	       "static SCM\n", dname,
 	       "(SCM rest)\n{\n",
 	       NIL);
-	Printf(df->code, "#define FUNC_NAME \"%s\"\n", proc_name);
 	Printf(df->code, "SCM argv[%d];\n", maxargs);
 	Printf(df->code, "int argc = SWIG_Guile_GetArgs (argv, rest, %d, %d, \"%s\");\n", 
 	       0, maxargs, proc_name);
 	Printv(df->code,dispatch,"\n",NIL);
 	Printf(df->code,"scm_misc_error(\"%s\", \"No matching method for generic function `%s'\", SCM_EOL);\n", proc_name, iname);
-	Printf(df->code, "#undef FUNC_NAME\n");
 	Printv(df->code,"}\n",NIL);
 	Wrapper_print(df,f_wrappers);
         if (use_scm_interface) {
@@ -1132,14 +1129,7 @@ public:
 	Printv(primitive_name, proc_name, NIL);
       Replaceall(method_signature, "_", "-");
       Replaceall(primitive_args, "_", "-");
-      if (!any_specialized_arg) {
-	/* If there would not be any specialized argument in
-	   the method declaration, we simply re-export the
-	   function.  This is a performance optimization. */
-	Printv(method_def, "(define ", goops_name, " ",
-	       primitive_name, ")\n", NIL);
-      }
-      else if (numreq == numargs) {
+      if (numreq == numargs) {
 	Printv(method_def, "(define-method (", goops_name, method_signature, ")\n", NIL);
 	Printv(method_def, "  (", primitive_name, primitive_args, "))\n", NIL);	
       }
@@ -1194,7 +1184,6 @@ public:
     Delete(doc_body);
     Delete(returns);
     Delete(tmp);
-    Delete(scheme_arg_names);
     DelWrapper(f);
     return SWIG_OK;
   }
@@ -1475,8 +1464,7 @@ public:
    * classDeclaration()
    * ------------------------------------------------------------ */
   virtual int classDeclaration(Node *n) {
-    String *class_name = NewStringf("<%s>", Getattr(n, "sym:name"));
-    Setattr(n, "guile:goopsclassname", class_name);
+    Setattr(known_classes, Getattr(n, "sym:name"), n);
     return Language::classDeclaration(n);
   }
 
@@ -1778,9 +1766,7 @@ public:
  * swig_guile()    - Instantiate module
  * ----------------------------------------------------------------------------- */
 
-static Language * new_swig_guile() {
+extern "C" Language *
+swig_guile(void) {
   return new GUILE();
 }
-extern "C" Language * swig_guile(void) {
-  return new_swig_guile();
-}
