@@ -12,7 +12,7 @@
  * See the file LICENSE for information on usage and redistribution.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_main_cxx[] = "$Header: /cvsroot/SWIG/Source/Modules/main.cxx,v 1.27 2003/12/16 03:43:51 beazley Exp $";
+char cvsroot_main_cxx[] = "$Header: /cvsroot/SWIG/Source/Modules/main.cxx,v 1.29 2004/01/10 00:04:34 cheetah Exp $";
 
 #if defined(_WIN32)
 #define WIN32_LEAN_AND_MEAN
@@ -25,11 +25,7 @@ char cvsroot_main_cxx[] = "$Header: /cvsroot/SWIG/Source/Modules/main.cxx,v 1.27
 #endif
 
 #include "swigwarn.h"
-
-extern "C" {
-#include "preprocessor.h"
-}
-
+#include "cparse.h"
 #include <ctype.h>
 
 // Global variables
@@ -52,7 +48,7 @@ extern "C" {
 extern String  *ModuleName;
 }
 
-static char *usage = (char*)"\
+static const char *usage1 = (const char*)"\
 \nGeneral Options\n\
      -c++            - Enable C++ processing\n\
      -co             - Check a file out of the SWIG library\n\
@@ -61,6 +57,8 @@ static char *usage = (char*)"\
      -E              - Preprocess only, does not generate wrapper code\n\
      -fcompact       - Compile in compact mode\n\
      -fvirtual       - Compile in virtual elimination mode\n\
+     -Fstandard      - Display error/warning messages in commonly used format\n\
+     -Fmicrosoft     - Display error/warning messages in Microsoft format\n\
      -help           - This output\n\
      -I<dir>         - Look for SWIG files in <dir>\n\
      -ignoremissing  - Ignore missing include files\n\
@@ -69,6 +67,9 @@ static char *usage = (char*)"\
      -l<ifile>       - Include SWIG library file <ifile>\n\
      -M              - List all dependencies \n\
      -MM             - List dependencies, but omit files in SWIG library\n\
+";
+// usage string split in two otherwise string is too big for some compilers
+static const char *usage2 = (const char*)"\
      -makedefault    - Create default constructors/destructors (the default)\n\
      -module <name>  - Set module name to <name>\n\
      -nocontract     - Turn off contract checking \n\
@@ -102,8 +103,7 @@ static String  *outdir = 0;
 // Checks the suffix of a file to see if we should emit extern declarations.
 // -----------------------------------------------------------------------------
 
-int
-check_suffix(char *name) {
+static int check_suffix(char *name) {
   char *c;
   if (!name) return 0;
   c = Swig_file_suffix(name);
@@ -123,8 +123,7 @@ check_suffix(char *name) {
 // Install all command line options as preprocessor symbols
 // ----------------------------------------------------------------------------- 
 
-static void
-install_opts(int argc, char *argv[]) {
+static void install_opts(int argc, char *argv[]) {
   int i;
   int noopt = 0;
   char *c;
@@ -201,16 +200,6 @@ void SWIG_config_cppext(const char *ext) {
   cpp_extension = (char *) ext;
 }
 
-extern  "C" Node *Swig_cparse(File *);
-extern  "C" void  Swig_cparse_cplusplus(int);
-extern  "C" void  Swig_cparse_debug_templates(int);
-extern  void Wrapper_virtual_elimination_mode_set(int);
-extern  void Wrapper_director_protected_mode_set(int);
-
-extern void Swig_contracts(Node *n);
-extern void Swig_contract_mode_set(int flag);
-extern int  Swig_contract_mode_get();
-
 int SWIG_main(int argc, char *argv[], Language *l) {
   int    i;
   char   *c;
@@ -222,7 +211,6 @@ int SWIG_main(int argc, char *argv[], Language *l) {
   int     tm_debug = 0;
   char   *includefiles[256];
   int     includecount = 0;
-  extern  int check_suffix(char *);
   int     dump_tags = 0;
   int     dump_tree = 0;
   int     browse = 0;
@@ -232,13 +220,8 @@ int SWIG_main(int argc, char *argv[], Language *l) {
   int     depend = 0;
   int     memory_debug = 0;
   int     allkw = 0;
-
   DOH    *libfiles = 0;
   DOH    *cpps = 0 ;
-  extern void Swig_browser(Node *n, int);
-  extern void Swig_default_allocators(Node *n);
-  extern void Swig_process_types(Node *n);
-
 
   /* Initialize the SWIG core */
   Swig_init();
@@ -506,8 +489,15 @@ int SWIG_main(int argc, char *argv[], Language *l) {
 	  } else if (strcmp(argv[i],"-dump_memory") == 0) {
 	    memory_debug =1;
 	    Swig_mark_arg(i);
+	  } else if (strcmp(argv[i],"-Fstandard") == 0) {
+            Swig_error_msg_format(EMF_STANDARD);
+	    Swig_mark_arg(i);
+	  } else if (strcmp(argv[i],"-Fmicrosoft") == 0) {
+            Swig_error_msg_format(EMF_MICROSOFT);
+	    Swig_mark_arg(i);
 	  } else if (strcmp(argv[i],"-help") == 0) {
-	    fputs(usage,stderr);
+	    fputs(usage1,stderr);
+	    fputs(usage2,stderr);
 	    Swig_mark_arg(i);
 	    help = 1;
 	  }
