@@ -8,10 +8,10 @@
  * can be used and distributed.
  *****************************************************************************/
 
-char cvsroot_ocaml_cxx[] = "$Header: /cvsroot/SWIG/Source/Modules/ocaml.cxx,v 1.39 2003/12/09 02:52:07 mmatus Exp $";
+char cvsroot_ocaml_cxx[] = "$Header: /cvsroot/SWIG/Source/Modules/ocaml.cxx,v 1.40 2003/12/23 20:31:06 arty Exp $";
 
 /***********************************************************************
- * $Header: /cvsroot/SWIG/Source/Modules/ocaml.cxx,v 1.39 2003/12/09 02:52:07 mmatus Exp $
+ * $Header: /cvsroot/SWIG/Source/Modules/ocaml.cxx,v 1.40 2003/12/23 20:31:06 arty Exp $
  *
  * ocaml.cxx
  *
@@ -30,6 +30,7 @@ static const char *usage = (char*)
     ("Ocaml Options (available with -ocaml)\n"
      "-prefix <name>  - Set a prefix <name> to be prepended to all names\n"
      "-where          - Emit library location\n"
+     "-suffix <name>  - Change .cxx to something else\n"
      "\n");
 
 static int classmode = 0;
@@ -114,6 +115,14 @@ public:
 		    } else {
 			Swig_arg_error();
 		    }
+		} else if (strcmp (argv[i], "-suffix") == 0) {
+		    if (argv[i + 1]) {
+			SWIG_config_cppext( argv[i+1] );
+			Swig_mark_arg (i);
+			Swig_mark_arg (i+1);
+			i++;
+		    } else
+			Swig_arg_error();
 		}
 	    }
 	}
