@@ -11,7 +11,7 @@
  * See the file LICENSE for information on usage and redistribution.	
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_overload_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/overload.cxx,v 1.9 2004/10/16 20:36:02 wsfulton Exp $";
+char cvsroot_overload_cxx[] = "$Header: /cvsroot/SWIG/Source/Modules/overload.cxx,v 1.5 2003/11/18 20:21:15 cheetah Exp $";
 
 #include "swigmod.h"
 
@@ -21,6 +21,14 @@ char cvsroot_overload_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/overlo
 String *argv_template_string;
 String *argc_template_string;
 
+/* -----------------------------------------------------------------------------
+ * Swig_overload_rank()
+ *
+ * This function takes an overloaded declaration and creates a list that ranks
+ * all overloaded methods in an order that can be used to generate a dispatch 
+ * function.
+ * ----------------------------------------------------------------------------- */
+
 struct Overloaded {
   Node      *n;          /* Node                               */
   int        argc;       /* Argument count                     */
@@ -28,25 +36,8 @@ struct Overloaded {
   int        error;      /* Ambiguity error                    */
 };
 
-/* -----------------------------------------------------------------------------
- * Swig_overload_rank()
- *
- * This function takes an overloaded declaration and creates a list that ranks
- * all overloaded methods in an order that can be used to generate a dispatch 
- * function.
- * Slight difference in the way this function is used by scripting languages and
- * statically typed languages. The script languages call this method via 
- * Swig_overload_dispatch() - where wrappers for all overloaded methods are generated,
- * however sometimes the code can never be executed. The non-scripting languages
- * call this method via Swig_overload_check() for each overloaded method in order
- * to determine whether or not the method should be wrapped. Note the slight
- * difference when overloading methods that differ by const only. The
- * scripting languages will ignore the const method, whereas the non-scripting
- * languages ignore the first method parsed.
- * ----------------------------------------------------------------------------- */
-
-static List *
-Swig_overload_rank(Node *n, bool script_lang_wrapping) {
+List *
+Swig_overload_rank(Node *n) {
   Overloaded  nodes[MAX_OVERLOAD];
   int         nnodes = 0;
   Node *o = Getattr(n,"sym:overloaded");
@@ -65,14 +56,12 @@ Swig_overload_rank(Node *n, bool script_lang_wrapping) {
       continue;
       } */
 
-    /* Make a list of all the declarations (methods) that are overloaded with
-     * this one particular method name */
     if (Getattr(c,"wrap:name")) {
       nodes[nnodes].n = c;
       nodes[nnodes].parms = Getattr(c,"wrap:parms");
       nodes[nnodes].argc = emit_num_required(nodes[nnodes].parms);
-      nodes[nnodes].error = 0;
-      nnodes++;
+	nodes[nnodes].error = 0;
+	nnodes++;
     }
     c = Getattr(c,"sym:nextSibling");
   }
@@ -116,13 +105,13 @@ Swig_overload_rank(Node *n, bool script_lang_wrapping) {
 	    if ((!t1) && (!nodes[i].error)) {
 	      Swig_warning(WARN_TYPEMAP_TYPECHECK, Getfile(nodes[i].n), Getline(nodes[i].n),
 			   "Overloaded %s(%s) not supported (no type checking rule for '%s').\n", 
-			   Getattr(nodes[i].n,"name"),ParmList_str_defaultargs(Getattr(nodes[i].n,"parms")),
+			   Getattr(nodes[i].n,"name"),ParmList_str(Getattr(nodes[i].n,"parms")),
 			   SwigType_str(Getattr(p1,"type"),0));
 	      nodes[i].error = 1;
 	    } else if ((!t2) && (!nodes[j].error)) {
 	      Swig_warning(WARN_TYPEMAP_TYPECHECK, Getfile(nodes[j].n), Getline(nodes[j].n),
 			   "Overloaded %s(%s) not supported (no type checking rule for '%s').\n", 
-			   Getattr(nodes[j].n,"name"),ParmList_str_defaultargs(Getattr(nodes[j].n,"parms")),
+			   Getattr(nodes[j].n,"name"),ParmList_str(Getattr(nodes[j].n,"parms")),
 			   SwigType_str(Getattr(p2,"type"),0));
 	      nodes[j].error = 1;
 	    }
@@ -202,46 +191,25 @@ Swig_overload_rank(Node *n, bool script_lang_wrapping) {
 	      if (Strcmp(dq1,dq2) == 0) {
 		
 		if (SwigType_isconst(d1) && !SwigType_isconst(d2)) {
-                  if (script_lang_wrapping) {
-                    // Swap nodes so that the const method gets ignored (shadowed by the non-const method)
-                    Overloaded t = nodes[i];
-                    nodes[i] = nodes[j];
-                    nodes[j] = t;
-                  }
+		  Overloaded t = nodes[i];
+		  nodes[i] = nodes[j];
+		  nodes[j] = t;
 		  differ = 1;
 		  if (!nodes[j].error) {
-                    if (script_lang_wrapping) {
-		      Swig_warning(WARN_LANG_OVERLOAD_CONST, Getfile(nodes[j].n), Getline(nodes[j].n),
-				   "Overloaded %s(%s) const ignored. Non-const method at %s:%d used.\n",
-				   Getattr(nodes[j].n,"name"), ParmList_protostr(nodes[j].parms),
-				   Getfile(nodes[i].n), Getline(nodes[i].n));
-                    } else {
-                      if (!Getattr(nodes[j].n, "overload:ignore"))
-		        Swig_warning(WARN_LANG_OVERLOAD_IGNORED, Getfile(nodes[j].n), Getline(nodes[j].n),
-				     "Overloaded method %s(%s) ignored. Method %s(%s) const at %s:%d used.\n",
-				     Getattr(nodes[j].n,"name"), ParmList_protostr(nodes[j].parms),
-			             Getattr(nodes[i].n,"name"), ParmList_protostr(nodes[i].parms),
-				     Getfile(nodes[i].n), Getline(nodes[i].n));
-                    }
+		    Swig_warning(WARN_LANG_OVERLOAD_CONST, Getfile(nodes[j].n), Getline(nodes[j].n),
+				 "Overloaded %s(%s) const ignored. Non-const method at %s:%d used.\n",
+				 Getattr(nodes[j].n,"name"), ParmList_protostr(nodes[j].parms),
+				 Getfile(nodes[i].n), Getline(nodes[i].n));
 		  }
 		  nodes[j].error = 1;
 		} else if (!SwigType_isconst(d1) && SwigType_isconst(d2)) {
 		  differ = 1;
 		  if (!nodes[j].error) {
-                    if (script_lang_wrapping) {
-		      Swig_warning(WARN_LANG_OVERLOAD_CONST, Getfile(nodes[j].n), Getline(nodes[j].n),
-				   "Overloaded %s(%s) const ignored. Non-const method at %s:%d used.\n",
-				   Getattr(nodes[j].n,"name"), ParmList_protostr(nodes[j].parms),
-				   Getfile(nodes[i].n), Getline(nodes[i].n));
-                    } else {
-                      if (!Getattr(nodes[j].n, "overload:ignore"))
-		        Swig_warning(WARN_LANG_OVERLOAD_IGNORED, Getfile(nodes[j].n), Getline(nodes[j].n),
-				     "Overloaded method %s(%s) const ignored. Method %s(%s) at %s:%d used.\n",
-				     Getattr(nodes[j].n,"name"), ParmList_protostr(nodes[j].parms),
-			             Getattr(nodes[i].n,"name"), ParmList_protostr(nodes[i].parms),
-				     Getfile(nodes[i].n), Getline(nodes[i].n));
-                    }
-                  }
+		    Swig_warning(WARN_LANG_OVERLOAD_CONST, Getfile(nodes[j].n), Getline(nodes[j].n),
+				 "Overloaded %s(%s) const ignored. Non-const method at %s:%d used.\n",
+				 Getattr(nodes[j].n,"name"), ParmList_protostr(nodes[j].parms),
+				 Getfile(nodes[i].n), Getline(nodes[i].n));
+		  }
 		  nodes[j].error = 1;
 		}
 	      }
@@ -251,24 +219,11 @@ Swig_overload_rank(Node *n, bool script_lang_wrapping) {
 	  }
 	  if (!differ) {
 	    if (!nodes[j].error) {
-              if (script_lang_wrapping) {
-	        Swig_warning(WARN_LANG_OVERLOAD_SHADOW, Getfile(nodes[j].n), Getline(nodes[j].n),
-			     "Overloaded %s(%s)%s is shadowed by %s(%s)%s at %s:%d.\n",
-			     Getattr(nodes[j].n,"name"), ParmList_protostr(nodes[j].parms),
-			     SwigType_isconst(Getattr(nodes[j].n,"decl")) ? " const" : "", 
-			     Getattr(nodes[i].n,"name"), ParmList_protostr(nodes[i].parms),
-			     SwigType_isconst(Getattr(nodes[i].n,"decl")) ? " const" : "", 
-			     Getfile(nodes[i].n),Getline(nodes[i].n));
-              } else {
-                if (!Getattr(nodes[j].n, "overload:ignore"))
-	          Swig_warning(WARN_LANG_OVERLOAD_IGNORED, Getfile(nodes[j].n), Getline(nodes[j].n),
-			       "Overloaded method %s(%s)%s ignored. Method %s(%s)%s at %s:%d used.\n",
-			       Getattr(nodes[j].n,"name"), ParmList_protostr(nodes[j].parms),
-			       SwigType_isconst(Getattr(nodes[j].n,"decl")) ? " const" : "", 
-                               Getattr(nodes[i].n,"name"), ParmList_protostr(nodes[i].parms),
-			       SwigType_isconst(Getattr(nodes[i].n,"decl")) ? " const" : "", 
-			       Getfile(nodes[i].n),Getline(nodes[i].n));
-              }
+	      Swig_warning(WARN_LANG_OVERLOAD_SHADOW, Getfile(nodes[j].n), Getline(nodes[j].n),
+			   "Overloaded %s(%s) is shadowed by %s(%s) at %s:%d.\n",
+			   Getattr(nodes[j].n,"name"), ParmList_protostr(nodes[j].parms),
+			   Getattr(nodes[i].n,"name"), ParmList_protostr(nodes[i].parms),
+			   Getfile(nodes[i].n),Getline(nodes[i].n));
 	      nodes[j].error = 1;
 	    }
 	  }
@@ -280,8 +235,6 @@ Swig_overload_rank(Node *n, bool script_lang_wrapping) {
   {
     int i;
     for (i = 0; i < nnodes; i++) {
-      if (nodes[i].error)
-        Setattr(nodes[i].n, "overload:ignore", "1");
       Append(result,nodes[i].n);
       //      Printf(stdout,"[ %d ] %s\n", i, ParmList_protostr(nodes[i].parms));
       //      Swig_print_node(nodes[i].n);
@@ -290,45 +243,6 @@ Swig_overload_rank(Node *n, bool script_lang_wrapping) {
   return result;
 }
 
-/* -----------------------------------------------------------------------------
- * print_typecheck()
- * ----------------------------------------------------------------------------- */
-
-static bool
-print_typecheck(String *f, int j, Parm *pj) {
-  char tmp[256];
-  sprintf(tmp,Char(argv_template_string),j);
-  String *tm = Getattr(pj,"tmap:typecheck");
-  if (tm) {
-    Replaceid(tm,Getattr(pj,"lname"),"_v");
-    Replaceall(tm,"$input", tmp);
-    Printv(f,tm,"\n",NIL);
-    return true;
-  }
-  else
-    return false;
-}
-
-/* -----------------------------------------------------------------------------
- * ReplaceFormat()
- * ----------------------------------------------------------------------------- */
-
-static String *
-ReplaceFormat (const String_or_char *fmt, int j) {
-  String *lfmt = NewString (fmt);
-  char buf[50];
-  sprintf (buf, "%d", j);
-  Replaceall (lfmt, "$numargs", buf);
-  int i;
-  String *commaargs = NewString ("");
-  for (i=0; i < j; i++) {
-    Printv (commaargs, ", ", NIL);
-    Printf (commaargs, Char(argv_template_string), i);
-  }
-  Replaceall (lfmt, "$commaargs", commaargs);
-  return lfmt;
-}
-
 /* -----------------------------------------------------------------------------
  * Swig_overload_dispatch()
  *
@@ -347,6 +261,38 @@ ReplaceFormat (const String_or_char *fmt, int j) {
  * the regular function arguments.
  * ----------------------------------------------------------------------------- */
 
+static bool print_typecheck(String *f, int j, Parm *pj)
+{
+  char tmp[256];
+  sprintf(tmp,Char(argv_template_string),j);
+  String *tm = Getattr(pj,"tmap:typecheck");
+  if (tm) {
+    Replaceid(tm,Getattr(pj,"lname"),"_v");
+    Replaceall(tm,"$input", tmp);
+    Printv(f,tm,"\n",NIL);
+    return true;
+  }
+  else
+    return false;
+}
+
+static String *
+ReplaceFormat (const String_or_char *fmt, int j)
+{
+  String *lfmt = NewString (fmt);
+  char buf[50];
+  sprintf (buf, "%d", j);
+  Replaceall (lfmt, "$numargs", buf);
+  int i;
+  String *commaargs = NewString ("");
+  for (i=0; i < j; i++) {
+    Printv (commaargs, ", ", NIL);
+    Printf (commaargs, Char(argv_template_string), i);
+  }
+  Replaceall (lfmt, "$commaargs", commaargs);
+  return lfmt;
+};
+
 String *
 Swig_overload_dispatch(Node *n, const String_or_char *fmt, int *maxargs) {
   int i,j;
@@ -356,7 +302,7 @@ Swig_overload_dispatch(Node *n, const String_or_char *fmt, int *maxargs) {
   String *f = NewString("");
 
   /* Get a list of methods ranked by precedence values and argument count */
-  List *dispatch = Swig_overload_rank(n, true);
+  List *dispatch = Swig_overload_rank(n);
   int   nfunc = Len(dispatch);
 
   /* Loop over the functions */
@@ -420,11 +366,3 @@ Swig_overload_dispatch(Node *n, const String_or_char *fmt, int *maxargs) {
   Delete(dispatch);
   return f;
 }
-
-/* -----------------------------------------------------------------------------
- * Swig_overload_check()
- * ----------------------------------------------------------------------------- */
-void Swig_overload_check(Node *n) {
-    Swig_overload_rank(n, false);
-}
-
