@@ -13,15 +13,20 @@
  *
  */
 
-char cvsroot_php4_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/php4.cxx,v 1.10 2004/12/01 00:50:06 marcelomatus Exp $";
+char cvsroot_php4_cxx[] = "$Header: /cvsroot/SWIG/Source/Modules/php4.cxx,v 1.6 2003/12/17 19:09:17 beazley Exp $";
 
 #include "swigmod.h"
 
+#ifndef MACSWIG
+#include "swigconfig.h"
+#endif
+
 #include <ctype.h>
 
 
 static const char *usage = (char*)"\
 PHP4 Options (available with -php4)\n\
+     -ldflags        - Print runtime libraries to link with\n\
      -cppext         - cpp file extension (default to .cpp)\n\
      -noproxy        - Don't generate proxy classes.\n\
      -dlname <name>  - Set module prefix to <name>\n\
@@ -310,6 +315,9 @@ public:
 	  Swig_mark_arg(i);
 	} else if(strcmp(argv[i], "-help") == 0) {
 	  fputs(usage, stderr);
+	} else if (strcmp (argv[i], "-ldflags") == 0) {
+	  printf("%s\n", SWIG_PHP_RUNTIME);
+	  SWIG_exit (EXIT_SUCCESS);
 	}
       }
     }
@@ -383,7 +391,7 @@ public:
       }
       
       Printf(f_extra,
-	     "# $Id: php4.cxx,v 1.10 2004/12/01 00:50:06 marcelomatus Exp $\n\n"
+	     "# $Id: php4.cxx,v 1.6 2003/12/17 19:09:17 beazley Exp $\n\n"
 	     "LTLIBRARY_NAME          = php_%s.la\n",
 	     module);
       
@@ -429,7 +437,7 @@ public:
       }
       
       Printf(f_extra,
-	     "dnl $Id: php4.cxx,v 1.10 2004/12/01 00:50:06 marcelomatus Exp $\n"
+	     "dnl $Id: php4.cxx,v 1.6 2003/12/17 19:09:17 beazley Exp $\n"
 	     "dnl ***********************************************************************\n"
 	     "dnl ** THIS config.m4 is provided for PHPIZE and PHP's consumption NOT\n"
 	     "dnl ** for any part of the rest of the %s build system\n"
@@ -886,7 +894,7 @@ public:
 */
 
   void create_command(char *cname, char *iname) {
-//    char *lower_cname = Swig_copy_string(cname);
+//    char *lower_cname = strdup(cname);
 //    char *c;
     
 //    for(c = lower_cname; *c != '\0'; c++) {
@@ -941,7 +949,7 @@ public:
       Putc(toupper((int )*iname), member_function_name);
       Printf(member_function_name, "%s", iname+1);
 
-      cpp_func(l, php_function_name);
+      cpp_func(Char(member_function_name), d, l, php_function_name);
      
       Delete(php_function_name);
       Delete(member_function_name);
@@ -1405,11 +1413,8 @@ public:
    * ------------------------------------------------------------ */
 
   virtual int classDeclaration(Node *n) {
-    if (!Getattr(n,"feature:onlychildren")) {
-      String *symname = Getattr(n,"sym:name");
-      Setattr(n,"php:proxy",symname);
-    }
-    
+    String *symname = Getattr(n,"sym:name");
+    Setattr(n,"php:proxy",symname);
     return Language::classDeclaration(n);
   }
   
@@ -1702,6 +1707,7 @@ public:
   virtual int memberfunctionHandler(Node *n) {
     char *name = GetChar(n, "name");
     char *iname = GetChar(n, "sym:name");
+    SwigType *t = Getattr(n, "type");
     ParmList *l = Getattr(n, "parms");
 
     this->Language::memberfunctionHandler(n);
@@ -1710,7 +1716,7 @@ public:
       char *realname = iname ? iname : name;
       String *php_function_name = Swig_name_member(shadow_classname, realname);
 
-      cpp_func(l, realname, php_function_name);
+      cpp_func(iname, t, l, realname, php_function_name);
 
     }
     return SWIG_OK;
@@ -1746,7 +1752,7 @@ public:
       static_flag = 1;      
       char *realname = iname ? iname : name;
       String *php_function_name = Swig_name_member(shadow_classname, realname);
-      cpp_func(Getattr(n, "parms"), symname, php_function_name);
+      cpp_func(Char(symname), Getattr(n, "type"), Getattr(n, "parms"), symname, php_function_name);
       static_flag = 0;
     }
 
@@ -1781,7 +1787,7 @@ public:
     if(Getattr(n,"feature:immutable")) {
       const_flag = 1;
     }
-    cpp_func(0, iname);
+    cpp_func(iname, d, 0, iname);
     static_flag = 0;
 
     create_command(iname, Char(Swig_name_wrapper(iname)));
@@ -2045,7 +2051,7 @@ public:
   }
 
   // This method is quite stale and ought to be factored out
-  void cpp_func(ParmList *l, String *php_function_name, String *handler_name = NULL) {
+  void cpp_func(char *iname, SwigType *t, ParmList *l, String *php_function_name, String *handler_name = NULL) {
     if(!shadow) return;
 
 	// if they didn't provide a handler name, use the realname
@@ -2110,7 +2116,8 @@ void typetrace(SwigType *ty, String *mangled, String *clientdata) {
   if (r_prevtracefunc) (*r_prevtracefunc)(ty, mangled, (String *) clientdata);
 }
 
-static Language * new_swig_php() {
+extern "C" Language *
+swig_php(void) {
   maininstance=new PHP4();
   if (! r_prevtracefunc) {
     r_prevtracefunc=SwigType_remember_trace(typetrace);
@@ -2120,6 +2127,4 @@ static Language * new_swig_php() {
   }
   return maininstance;
 }
-extern "C" Language * swig_php(void) {
-  return new_swig_php();
-}
+
