@@ -1,7 +1,7 @@
 /********************************************************************
  * Ruby module for SWIG
  *
- * $Header: /cvsroot/swig/SWIG/Source/Modules/ruby.cxx,v 1.71 2004/12/13 22:41:13 wsfulton Exp $
+ * $Header: /cvsroot/SWIG/Source/Modules/ruby.cxx,v 1.47 2003/12/22 17:23:02 ljohnson Exp $
  *
  * Copyright (C) 2000  Network Applied Communication Laboratory, Inc.
  * Copyright (C) 2000  Information-technology Promotion Agency, Japan
@@ -10,11 +10,15 @@
  *
  ********************************************************************/
 
-char cvsroot_ruby_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/ruby.cxx,v 1.71 2004/12/13 22:41:13 wsfulton Exp $";
+char cvsroot_ruby_cxx[] = "$Header: /cvsroot/SWIG/Source/Modules/ruby.cxx,v 1.47 2003/12/22 17:23:02 ljohnson Exp $";
 
 #include "swigmod.h"
 #define SWIG_PROTECTED_TARGET_METHODS 1
 
+#ifndef MACSWIG
+#include "swigconfig.h"
+#endif
+
 #include <ctype.h>
 #include <string.h>
 #include <limits.h> /* for INT_MAX */
@@ -119,22 +123,22 @@ char cvsroot_ruby_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/ruby.cxx,v
 static const char *
 usage = "\
 Ruby Options (available with -ruby)\n\
+     -ldflags        - Print runtime libraries to link with\n\
      -globalmodule   - Wrap everything into the global module\n\
      -minherit       - Attempt to support multiple inheritance\n\
-     -prefix <name>  - Set a prefix <name> to be prepended to all names\n\
      -feature <name> - Set feature name to <name> (used by `require')\n";
 
 
 #define RCLASS(hash, name) (RClass*)(Getattr(hash, name) ? Data(Getattr(hash, name)) : 0)
 #define SET_RCLASS(hash, name, klass) Setattr(hash, name, NewVoid(klass, 0))
 
+
 class RUBY : public Language {
 private:
 
   String *module;
   String *modvar;
   String *feature;
-  String *prefix;
   int current;
   Hash *classes;		/* key=cname val=RClass */
   RClass *klass;		/* Currently processing class */
@@ -148,6 +152,7 @@ private:
   File *f_wrappers;
   File *f_init;
 
+  bool use_kw;
   bool useGlobalModule;
   bool multipleInheritance;
 
@@ -176,7 +181,6 @@ public:
     module = 0;
     modvar = 0;
     feature = 0;
-    prefix = 0;
     current = NO_CPP;
     classes = 0;
     klass = 0;
@@ -185,18 +189,9 @@ public:
     f_header = 0;
     f_wrappers = 0;
     f_init = 0;
+    use_kw = false;
     useGlobalModule = false;
     multipleInheritance = false;
-    director_prot_ctor_code = NewString("");    
-    Printv(director_prot_ctor_code,
-	   "if ( $comparison ) { /* subclassed */\n",
-	   "  $director_new \n",
-	   "} else {\n",
-	   "  rb_raise(rb_eRuntimeError,\"accessing abstract class or protected constructor\"); \n",
-	   "  return Qnil;\n",
-	   "}\n", NIL);
-    director_multiple_inheritance = 0;
-    director_language = 1;
   }
   
   /* ---------------------------------------------------------------------
@@ -228,20 +223,12 @@ public:
 	  Swig_mark_arg(i);
 	} else if (strcmp(argv[i],"-minherit") == 0) {
           multipleInheritance = true;
-	  director_multiple_inheritance = 1;
 	  Swig_mark_arg(i);
-        } else if (strcmp(argv[i],"-prefix") == 0) {
-          if (argv[i+1]) {
-            char *name = argv[i+1];
-            prefix = NewString(name);
-            Swig_mark_arg(i);
-            Swig_mark_arg(i+1);
-            i++;
-          } else {
-            Swig_arg_error();
-          }
 	} else if (strcmp(argv[i],"-help") == 0) {
 	  Printf(stderr,"%s\n", usage);
+	} else if (strcmp (argv[i],"-ldflags") == 0) {
+	  printf("%s\n", SWIG_RUBY_RUNTIME);
+	  SWIG_exit (EXIT_SUCCESS);
 	}
       }
     }
@@ -357,15 +344,12 @@ public:
         }
         if (Getattr(options, "ruby_minherit")) {
           multipleInheritance = true;
-	  director_multiple_inheritance = 1;
         }
       }
     }
 
     /* Set comparison with none for ConstructorToFunction */
-
-    
-    setSubclassInstanceCheck(NewStringf("strcmp(rb_obj_classname(self), classname) != 0")); 
+    setSubclassInstanceCheck(NewString("CLASS_OF(self) != Qnil")); // FIXME
     // setSubclassInstanceCheck(NewString("CLASS_OF(self) != cFoo.klass"));
 
     /* Initialize all of the output files */
@@ -409,6 +393,10 @@ public:
 
     Swig_banner(f_runtime);
 
+    if (NoInclude) {
+      Printf(f_runtime, "#define SWIG_NOINCLUDE\n");
+    }
+
     if (directorsEnabled()) {
       Printf(f_runtime,"#define SWIG_DIRECTORS\n");
     }
@@ -500,29 +488,7 @@ public:
   virtual int importDirective(Node *n) {
     String *modname = Getattr(n,"module");
     if (modname) {
-      if (prefix) {
-        Insert(modname, 0, prefix);
-      }
-      
-      List *modules = Split(modname,':',INT_MAX);
-      if (modules && Len(modules) > 0) {
-        modname = NewString("");
-        String *last = NULL;
-        Iterator m = First(modules);
-        while (m.item) {
-          if (Len(m.item) > 0) {
-            if (last) {
-              Append(modname, "/");
-            }
-            Append(modname, m.item);
-            last = m.item;
-          }
-          m = Next(m);
-        }
-        Printf(f_init,"rb_require(\"%s\");\n", modname);
-        Delete(modname);
-      }
-      Delete(modules);
+      Printf(f_init,"rb_require(\"%s\");\n", modname);
     }
     return Language::importDirective(n);
   }
@@ -539,10 +505,6 @@ public:
     if (module == 0) {
       /* Start with the empty string */
       module = NewString("");
-
-      if (prefix) {
-        Insert(mod_name, 0, prefix);
-      }
       
       /* Account for nested modules */
       List *modules = Split(mod_name,':',INT_MAX);
@@ -636,15 +598,15 @@ public:
     String *s = NewString("");
     String *temp = NewString("");
     
-#ifdef SWIG_PROTECTED_TARGET_METHODS
-    const char* rb_define_method = is_public(n) ?
-      "rb_define_method" : "rb_define_protected_method" ;
-#else        
-    const char* rb_define_method = "rb_define_method";
-#endif	
     switch (current) {
     case MEMBER_FUNC: 
       {
+#ifdef SWIG_PROTECTED_TARGET_METHODS
+	const char* rb_define_method = is_protected(n) ?
+	  "rb_define_protected_method" : "rb_define_method";
+#else        
+	const char* rb_define_method = "rb_define_method";
+#endif	
 	if (multipleInheritance) {
 	  Printv(klass->init, tab4, rb_define_method,"(", klass->mImpl, ", \"",
 		 iname, "\", ", wname, ", -1);\n", NIL);
@@ -659,7 +621,7 @@ public:
       Replaceall(klass->init,"$allocator", s);
       break;
     case CONSTRUCTOR_INITIALIZE:
-      Printv(s, tab4, rb_define_method,"(", klass->vname,
+      Printv(s, tab4, "rb_define_method(", klass->vname,
 	     ", \"initialize\", ", wname, ", -1);\n", NIL);
       Replaceall(klass->init,"$initializer", s);
       break;
@@ -757,7 +719,7 @@ public:
     source = NewString("");
     target = NewString("");
 
-    bool ctor_director = (current == CONSTRUCTOR_INITIALIZE && Swig_directorclass(n));
+    bool use_director = (current == CONSTRUCTOR_INITIALIZE && Swig_directorclass(n));
 
     /**
      * The 'start' value indicates which of the C/C++ function arguments
@@ -766,7 +728,7 @@ public:
      * the first argument (with name arg1) is based on the value of argv[0].
      * If start is one, then arg1 is based on the value of argv[1].
      */
-    int start = (current == MEMBER_FUNC || current == MEMBER_VAR || ctor_director) ? 1 : 0;
+    int start = (current == MEMBER_FUNC || current == MEMBER_VAR || use_director) ? 1 : 0;
 
     int varargs = emit_isvarargs(l);
 
@@ -1008,10 +970,10 @@ public:
     int  numarg = emit_num_arguments(l);
     int  numreq = emit_num_required(l);
     int  varargs = emit_isvarargs(l);
-    bool allow_kwargs = Getattr(n,"feature:kwargs") ? true : false;
+    bool allow_kwargs = use_kw || Getattr(n,"feature:kwargs");
     
-    bool ctor_director = (current == CONSTRUCTOR_INITIALIZE && Swig_directorclass(n));
-    int start = (current == MEMBER_FUNC || current == MEMBER_VAR || ctor_director) ? 1 : 0;
+    bool use_director = (current == CONSTRUCTOR_INITIALIZE && Swig_directorclass(n));
+    int start = (current == MEMBER_FUNC || current == MEMBER_VAR || use_director) ? 1 : 0;
 
     /* Now write the wrapper function itself */
     if        (current == CONSTRUCTOR_ALLOCATE) {
@@ -1045,7 +1007,7 @@ public:
     }
 
     // FIXME?
-    if (ctor_director) {
+    if (use_director) {
       numarg--;
       numreq--;
     }
@@ -1081,8 +1043,7 @@ public:
 
     if (directorsEnabled()) {
       if (!is_smart_pointer()) {
-        if (/*directorbase &&*/ !constructor && !destructor 
-	    && isVirtual  && !Getattr(n,"feature:nodirector")) {
+        if (/*directorbase &&*/ !constructor && !destructor && isVirtual) {
           Wrapper_add_local(f, "director", "Swig::Director *director = 0");
           Printf(f->code, "director = dynamic_cast<Swig::Director *>(arg1);\n");
           Printf(f->code, "if (director && (director->swig_get_self() == self)) director->swig_set_up();\n");
@@ -1129,8 +1090,10 @@ public:
           int is_reference = SwigType_isreference_return(decl);
           if (is_pointer || is_reference) {
             String *type = Getattr(n, "type");
-            Node *modname = Getattr(parent, "module");
-            Node *target = Swig_directormap(modname, type);
+            //Node *classNode = Swig_methodclass(n);
+            //Node *module = Getattr(classNode, "module");
+            Node *module = Getattr(parent, "module");
+            Node *target = Swig_directormap(module, type);
             if (target) unwrap = true;
           }
           if (unwrap) {
@@ -1171,7 +1134,7 @@ public:
       Printv(f->code,cleanup,NIL);
 
     /* Look for any remaining cleanup.  This processes the %new directive */
-    if (current != CONSTRUCTOR_ALLOCATE && Getattr(n, "feature:new")) {
+    if (Getattr(n, "feature:new")) {
       tm = Swig_typemap_lookup_new("newfree",n,"result",0);
       if (tm) {
 	Replaceall(tm,"$source","result");
@@ -1259,16 +1222,14 @@ public:
 	   NIL);
     
     Wrapper_add_local(f, "argc", "int argc");
-    bool ctor_director = (current == CONSTRUCTOR_INITIALIZE && Swig_directorclass(n));
-    if (current == MEMBER_FUNC || current == MEMBER_VAR || ctor_director) {
+    if (current == MEMBER_FUNC || current == MEMBER_VAR) {
       Printf(tmp, "VALUE argv[%d]", maxargs+1);
     } else {
       Printf(tmp, "VALUE argv[%d]", maxargs);
     }
     Wrapper_add_local(f, "argv", tmp);
     Wrapper_add_local(f, "ii", "int ii");
-
-    if (current == MEMBER_FUNC || current == MEMBER_VAR || ctor_director) {
+    if (current == MEMBER_FUNC || current == MEMBER_VAR) {
       Printf(f->code, "argc = nargs + 1;\n");
       Printf(f->code, "argv[0] = self;\n");
       Printf(f->code, "for (ii = 1; (ii < argc) && (ii < %d); ii++) {\n", maxargs);
@@ -1439,7 +1400,7 @@ public:
     }
     
     Swig_warning(WARN_RUBY_WRONG_NAME, input_file, line_number,
-		 "Wrong %s name %s\n", reason, name);
+		 "Wrong %s name\n", reason);
     
     return name;
   }
@@ -1505,24 +1466,22 @@ public:
    * ----------------------------------------------------------------------------- */
 
   virtual int classDeclaration(Node *n) {
-    if (!Getattr(n,"feature:onlychildren")) {
-      String *name = Getattr(n,"name");
-      String *symname = Getattr(n,"sym:name");
-      String *tdname = Getattr(n,"tdname");
-    
-      name = tdname ? tdname : name;
-      String *namestr = SwigType_namestr(name);
-      klass = RCLASS(classes, Char(namestr));
-      if (!klass) {
-	klass = new RClass();
-	String *valid_name = NewString(symname ? symname : namestr);
-	validate_const_name(Char(valid_name), "class");
-	klass->set_name(namestr, symname, valid_name);
-	SET_RCLASS(classes, Char(namestr), klass);
-	Delete(valid_name);
-      }
-      Delete(namestr);
-    }    
+    String *name = Getattr(n,"name");
+    String *symname = Getattr(n,"sym:name");
+    String *tdname = Getattr(n,"tdname");
+  
+    name = tdname ? tdname : name;
+    String *namestr = SwigType_namestr(name);
+    klass = RCLASS(classes, Char(namestr));
+    if (!klass) {
+      klass = new RClass();
+      String *valid_name = NewString(symname ? symname : namestr);
+      validate_const_name(Char(valid_name), "class");
+      klass->set_name(namestr, symname, valid_name);
+      SET_RCLASS(classes, Char(namestr), klass);
+      Delete(valid_name);
+    }
+    Delete(namestr);
     return Language::classDeclaration(n);
   }
 
@@ -1726,47 +1685,12 @@ public:
    * Method for adding C++ member constructor
    * -------------------------------------------------------------------- */
 
-  void set_director_ctor_code(Node *n) 
-  {
-    /* director ctor code is specific for each class */
-    Delete(director_prot_ctor_code);
-    director_prot_ctor_code = NewString("");
-    Node *pn = Swig_methodclass(n);
-    String *symname = Getattr(pn,"sym:name");
-    String  *name   = Copy(symname);
-    char *cname = Char(name);
-    if(cname) cname[0] = toupper(cname[0]);
-    Printv(director_prot_ctor_code,
-	   "char *classname = \"",module,"::",cname,"\";\n",
-	   "if ( $comparison ) { /* subclassed */\n",
-	   "  $director_new \n",
-	   "} else {\n",
-	   "  rb_raise(rb_eNameError,\"accessing abstract class or protected constructor\"); \n",
-	   "  return Qnil;\n",
-	   "}\n", NIL);
-    Delete(director_ctor_code);
-    director_ctor_code = NewString("");
-    Printv(director_ctor_code,
-	   "char *classname = \"",module,"::",cname,"\";\n",
-	   "if ( $comparison ) { /* subclassed */\n",
-	   "  $director_new \n",
-	   "} else {\n",
-	   "  $nondirector_new \n",
-	   "}\n", NIL);
-    Delete(name);
-  }
-
   virtual int constructorHandler(Node *n) {
     int   use_director = Swig_directorclass(n);
-    if (use_director) {
-      set_director_ctor_code(n);
-    }
-    
+
     /* First wrap the allocate method */
     current = CONSTRUCTOR_ALLOCATE;
     Swig_name_register((String_or_char *) "construct", (String_or_char *) "%c_allocate");
-    
-    
     Language::constructorHandler(n);
 
     /* 
@@ -1806,20 +1730,6 @@ public:
     return SWIG_OK;
   }
 
-  virtual int copyconstructorHandler(Node *n) {
-    int   use_director = Swig_directorclass(n);
-    if (use_director) {
-      set_director_ctor_code(n);
-    }
-
-    /* First wrap the allocate method */
-    current = CONSTRUCTOR_ALLOCATE;
-    Swig_name_register((String_or_char *) "construct", (String_or_char *) "%c_allocate");
-    
-    return Language::copyconstructorHandler(n);
-  }
-  
-
   /* ---------------------------------------------------------------------
    * destructorHandler()
    * -------------------------------------------------------------------- */
@@ -1848,18 +1758,13 @@ public:
       /*    Printv(freebody, Swig_name_destroy(name), "(", Swig_cparm_name(0,0), ")", NIL); */
       Printv(freebody,Getattr(n,"wrap:action"), NIL);
     } else {
-      String  *action = Getattr(n,"wrap:action");
-      if (action) {
-	Printv(freebody, action, NIL);
-      } else {      
-	/* In the case swig emits no destroy function. */
-	if (CPlusPlus)
-	  Printf(freebody, "delete %s;\n", Swig_cparm_name(0,0));
-	else
-	  Printf(freebody, "free((char*) %s);\n", Swig_cparm_name(0,0));
-      }
+      /* When no extend mode, swig emits no destroy function. */
+      if (CPlusPlus)
+	Printf(freebody, "delete %s", Swig_cparm_name(0,0));
+      else
+	Printf(freebody, "free((char*) %s)", Swig_cparm_name(0,0));
     }
-    Printv(freebody, "}\n", NIL);
+    Printv(freebody, ";\n}\n", NIL);
   
     Replaceall(klass->header,"$freeproto", freeproto);
     Printv(f_wrappers, freebody, NIL);
@@ -1942,6 +1847,10 @@ public:
     return Language::classDirectorEnd(n);
   }
   
+  virtual int unrollVirtualMethods(Node *n, Node *parent, Hash *vm, int default_director, int &virtual_destructor) {
+    return Language::unrollVirtualMethods(n, parent, vm, default_director, virtual_destructor);
+  }
+  
   /* ------------------------------------------------------------
    * classDirectorConstructor()
    * ------------------------------------------------------------ */
@@ -1954,36 +1863,40 @@ public:
     String *classname = NewString("");
     Printf(classname, "SwigDirector_%s", supername);
 
-    /* insert self parameter */
-    Parm *p;
+    /* insert self and disown parameters */
+    Parm *p, *ip;
     ParmList *superparms = Getattr(n, "parms");
     ParmList *parms = CopyParmList(superparms);
     String *type = NewString("VALUE");
     p = NewParm(type, NewString("self"));
     set_nextSibling(p, parms);
     parms = p;
+    for (ip = parms; nextSibling(ip); ) ip = nextSibling(ip);
+    p = NewParm(NewString("bool"), NewString("disown"));
+    Setattr(p, "arg:byname", "1");
+    Setattr(n, "director:postfix_args", p);
+    Setattr(p, "value", "0");
+    set_nextSibling(ip, p);
     
-    if (!Getattr(n,"defaultargs")) {
-      /* constructor */
-      {
-        Wrapper *w = NewWrapper();
-        String *call;
-        String *basetype = Getattr(parent, "classtype");
-        String *target = Swig_method_decl(decl, classname, parms, 0, 0);
-        call = Swig_csuperclass_call(0, basetype, superparms);
-        Printf(w->def, "%s::%s: %s, Swig::Director(self) { }", classname, target, call);
-        Delete(target);
-        Wrapper_print(w, f_directors);
-        Delete(call);
-        DelWrapper(w);
-      }
-      
-      /* constructor header */
-      {
-        String *target = Swig_method_decl(decl, classname, parms, 0, 1);
-        Printf(f_directors_h, "    %s;\n", target);
-        Delete(target);
-      }
+    /* constructor */
+    {
+      Wrapper *w = NewWrapper();
+      String *call;
+      String *basetype = Getattr(parent, "classtype");
+      String *target = Swig_method_decl(decl, classname, parms, 0, 0);
+      call = Swig_csuperclass_call(0, basetype, superparms);
+      Printf(w->def, "%s::%s: %s, Swig::Director(self, disown) { }", classname, target, call);
+      Delete(target);
+      Wrapper_print(w, f_directors);
+      Delete(call);
+      DelWrapper(w);
+    }
+    
+    /* constructor header */
+    {
+      String *target = Swig_method_decl(decl, classname, parms, 0, 1);
+      Printf(f_directors_h, "    %s;\n", target);
+      Delete(target);
     }
 
     Delete(sub);
@@ -1993,25 +1906,21 @@ public:
     return Language::classDirectorConstructor(n);
   }
   
-  /* ------------------------------------------------------------
-   * classDirectorDefaultConstructor()
-   * ------------------------------------------------------------ */
-
   virtual int classDirectorDefaultConstructor(Node *n) {
     String *classname;
     Wrapper *w;
     classname = Swig_class_name(n);
     w = NewWrapper();
-    Printf(w->def, "SwigDirector_%s::SwigDirector_%s(VALUE self) : Swig::Director(self) { }", classname, classname);
+    Printf(w->def, "SwigDirector_%s::SwigDirector_%s(VALUE self, bool disown) : Swig::Director(self, disown) { }", classname, classname);
     Wrapper_print(w, f_directors);
     DelWrapper(w);
-    Printf(f_directors_h, "    SwigDirector_%s(VALUE self);\n", classname);
+    Printf(f_directors_h, "    SwigDirector_%s(VALUE self, bool disown = true);\n", classname);
     Delete(classname);
     return Language::classDirectorDefaultConstructor(n);
   }
   
   /* ---------------------------------------------------------------
-   * exceptionSafeMethodCall()
+   * classDirectorMethod()
    *
    * Emit a virtual director method to pass a method call on to the 
    * underlying Ruby instance.
@@ -2052,7 +1961,7 @@ public:
       // Exception handler
       Printf(rescue->def, "VALUE %s(VALUE args, VALUE error) {\n", rescueName); 
       Replaceall(tm, "$error", "error");
-      Printf(rescue->code, "if (%s == 1) ", depthCountName);
+      Printf(rescue->code, "if (%s == 0) ", depthCountName);
       Printv(rescue->code, Str(tm), "\n", NIL);
       Printf(rescue->code, "%s--;\n", depthCountName);
       Printv(rescue->code, "rb_exc_raise(error);\n", NIL);
@@ -2060,28 +1969,21 @@ public:
       
       // Main code
       Wrapper_add_localv(w, "args", "Swig::body_args", "args", NIL);
-      Wrapper_add_localv(w, "status", "int", "status", NIL);
       Printv(w->code, "args.recv = swig_get_self();\n", NIL);
       Printf(w->code, "args.id = rb_intern(\"%s\");\n", methodName);
       Printf(w->code, "args.argc = %d;\n", argc);
       if (argc > 0) {
         Wrapper_add_localv(w, "i", "int", "i", NIL);
         Printf(w->code, "args.argv = new VALUE[%d];\n", argc);
-        for (int i = 0; i < argc; i++) {
-          Printf(w->code, "args.argv[%d] = obj%d;\n", i, i);
-        } 
+        Printf(w->code, "for (i = 0; i < %d; i++) {\n", argc);
+        Printv(w->code, "args.argv[i] = Qnil;\n", NIL);
+        Printv(w->code, "}\n", NIL);
       } else {
         Printv(w->code, "args.argv = 0;\n", NIL);
       }
       Printf(w->code,
-        "result = rb_protect(PROTECTFUNC(%s), reinterpret_cast<VALUE>(&args), &status);\n", bodyName, rescueName);
-      Printf(w->code,
-        "if (status) {\n");
-      Printf(w->code,
-        "VALUE lastErr = rb_gv_get(\"$!\");\n");
-      Printf(w->code,
-        "%s(reinterpret_cast<VALUE>(&args), lastErr);\n", rescueName);
-      Printf(w->code, "}\n");
+        "result = rb_rescue2((VALUE(*)(ANYARGS)) %s, reinterpret_cast<VALUE>(&args), (VALUE(*)(ANYARGS)) %s, reinterpret_cast<VALUE>(&args), rb_eStandardError, 0);\n",
+        bodyName, rescueName);
       if (argc > 0) {
         Printv(w->code, "delete [] args.argv;\n", NIL);
       }
@@ -2163,44 +2065,13 @@ public:
     String *qualified_name = NewStringf("%s::%s", pclassname, name);
     target = Swig_method_decl(decl, qualified_name, l, 0, 0);
     String *rtype = SwigType_str(type, 0);
-    Printf(w->def, "%s %s", rtype, target);
+    Printf(w->def, "%s %s {", rtype, target);
     Delete(qualified_name);
     Delete(target);
     /* header declaration */
     target = Swig_method_decl(decl, name, l, 0, 1);
-    Printf(declaration, "    virtual %s %s", rtype, target);
+    Printf(declaration, "    virtual %s %s;\n", rtype, target);
     Delete(target);
-
-    // Get any exception classes in the throws typemap
-    ParmList *throw_parm_list = 0;
-
-    if ((throw_parm_list = Getattr(n,"throws")) || Getattr(n,"throw")) {
-      Parm      *p;
-      int       gencomma = 0;
-
-      Append(w->def, " throw(");
-      Append(declaration, " throw(");
-
-      if (throw_parm_list) Swig_typemap_attach_parms("throws", throw_parm_list, 0);
-      for (p = throw_parm_list; p; p=nextSibling(p)) {
-        if ((tm = Getattr(p,"tmap:throws"))) {
-          if (gencomma++) {
-            Append(w->def, ", ");
-            Append(declaration, ", ");
-          }
-
-          Printf(w->def, "%s", SwigType_str(Getattr(p, "type"),0));
-          Printf(declaration, "%s", SwigType_str(Getattr(p, "type"),0));
-        }
-      }
-
-      Append(w->def, ")");
-      Append(declaration, ")");
-    }
-
-    Append(w->def, " {");
-    Append(declaration, ";\n");
-
     
     /* attach typemaps to arguments (C/C++ -> Ruby) */
     String *arglist = NewString("");
@@ -2261,8 +2132,8 @@ public:
 	 * to SWIGTYPE) called DIRECTORTYPE?
 	 */
 	if (SwigType_ispointer(parameterType) || SwigType_isreference(parameterType)) {
-	  Node *modname = Getattr(parent, "module");
-	  Node *target = Swig_directormap(modname, parameterType);
+	  Node *module = Getattr(parent, "module");
+	  Node *target = Swig_directormap(module, parameterType);
 	  sprintf(source, "obj%d", idx++);
 	  String *nonconst = 0;
 	  /* strip pointer/reference --- should move to Swig/stype.c */
@@ -2424,23 +2295,19 @@ public:
 
     /* any existing helper functions to handle this? */
     if (!is_void) {
-      String* rettype = SwigType_str(return_type, 0);      
       if (!SwigType_isreference(return_type)) {
-	Printf(w->code, "return (%s) c_result;\n", rettype);
+        Printf(w->code, "return c_result;\n");
       } else {
-	Printf(w->code, "return (%s) *c_result;\n", rettype);
+        Printf(w->code, "return *c_result;\n");
       }
-      Delete(rettype);
     }
 
     Printf(w->code, "}\n");
 
     /* emit the director method */
     if (status == SWIG_OK) {
-      if (!Getattr(n,"defaultargs")) {
-        Wrapper_print(w, f_directors);
-        Printv(f_directors_h, declaration, NIL);
-      }
+      Wrapper_print(w, f_directors);
+      Printv(f_directors_h, declaration, NIL);
     }
 
     /* clean up */
@@ -2487,12 +2354,10 @@ public:
  * swig_ruby()    - Instantiate module
  * ----------------------------------------------------------------------------- */
 
-static Language * new_swig_ruby() {
+extern "C" Language *
+swig_ruby(void) {
   return new RUBY();
 }
-extern "C" Language * swig_ruby(void) {
-  return new_swig_ruby();
-}
 
 
 /*
