@@ -20,9 +20,12 @@
  * can be used and distributed.
  *******************************************************************************/
 
-char cvsroot_swigmain_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/swigmain.cxx,v 1.12 2004/10/05 21:02:50 marcelomatus Exp $";
+char cvsroot_swigmain_cxx[] = "$Header: /cvsroot/SWIG/Source/Modules/swigmain.cxx,v 1.8 2003/11/18 20:21:14 cheetah Exp $";
+
+#ifndef MACSWIG
+#include "swigconfig.h"
+#endif
 
-#include <ctype.h>
 #include "swigmod.h"
 
 /* Module factories.  These functions are used to instantiate 
@@ -37,7 +40,6 @@ extern "C" {
   Language *swig_perl5(void);
   Language *swig_ruby(void);
   Language *swig_guile(void);
-  Language *swig_modula3(void);
   Language *swig_mzscheme(void);
   Language *swig_java(void);
   Language *swig_php(void);
@@ -47,7 +49,6 @@ extern "C" {
   Language *swig_xml(void);
   Language *swig_chicken(void);
   Language *swig_csharp(void);
-  Language *swig_allegrocl(void);
 }
 
 struct swig_module {
@@ -61,12 +62,10 @@ struct swig_module {
    list sorted alphabetically. */
 
 swig_module  modules[] = {  
-  {"-allegrocl", swig_allegrocl, "ALLEGROCL"},
   {"-chicken",   swig_chicken,   "CHICKEN"},
   {"-csharp",    swig_csharp,    "C#"},
   {"-guile",     swig_guile,     "Guile"},
   {"-java",      swig_java,      "Java"},
-  {"-modula3",   swig_modula3,   "Modula 3"},
   {"-mzscheme",  swig_mzscheme,  "Mzscheme"},
   {"-ocaml",     swig_ocaml,     "Ocaml"},
   {"-perl",      swig_perl5,     "Perl"},
@@ -98,53 +97,12 @@ swig_module  modules[] = {
 // Main program.    Initializes the files and starts the parser.
 //-----------------------------------------------------------------
 
-void SWIG_merge_envopt(const char *env, int oargc, char *oargv[],
-		       int *nargc, char ***nargv)
-{
-  if (!env) {
-    *nargc = oargc;
-    *nargv = oargv;
-    return;
-  }
 
-  int argc = 1;
-  int arge = oargc + 1024;
-  char **argv = (char **) malloc(sizeof(char*)*(arge));
-  char *buffer = (char*)  malloc(2048);
-  char *b = buffer;
-  char *be = b + 1023;
-  const char *c = env;
-  while ((b != be) && *c && (argc < arge)) {
-    while (isspace(*c) && *c) ++c;
-    if (*c) {
-      argv[argc] = b;
-      ++argc;
-    }    
-    while ((b != be) && *c && !isspace(*c)) {
-      *(b++) = *(c++);
-    }
-    *b++ = 0;
-  }
-
-  argv[0] = oargv[0];
-  for (int i = 1; (i < oargc) && (argc < arge); ++i, ++argc) {
-    argv[argc] = oargv[i];
-  }
-
-  *nargc = argc;
-  *nargv = argv;
-}
-
-int main(int margc, char **margv) {
+int main(int argc, char **argv) {
   int i;
   Language *dl = 0;
   ModuleFactory fac = 0;
 
-  int argc;
-  char **argv;
-
-  SWIG_merge_envopt(getenv("SWIG_FEATURES"), margc, margv, &argc, &argv);
-
 #ifdef MACSWIG
   SIOUXSettings.asktosaveonclose = false;
   argc = ccommand(&argv);
