@@ -20,13 +20,18 @@
  *
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_tcl8_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/tcl8.cxx,v 1.15 2004/10/29 22:28:00 marcelomatus Exp $";
+char cvsroot_tcl8_cxx[] = "$Header: /cvsroot/SWIG/Source/Modules/tcl8.cxx,v 1.9 2003/11/25 03:51:35 beazley Exp $";
 
 #include "swigmod.h"
 
+#ifndef MACSWIG
+#include "swigconfig.h"
+#endif
+
 static const char *usage = (char*)"\
 Tcl 8 Options (available with -tcl)\n\
      -itcl           - Enable ITcl support\n\
+     -ldflags        - Print runtime libraries to link with\n\
      -nosafe         - Leave out SafeInit module function.\n\
      -prefix <name>  - Set a prefix <name> to be prepended to all names\n\
      -namespace      - Build module into a Tcl 8 namespace\n\
@@ -43,7 +48,6 @@ static int         nspace = 0;
 static String     *init_name = 0;
 static String     *ns_name = 0;
 static int         have_constructor;
-static String     *constructor_name;
 static int         have_destructor;
 static int         have_base_classes;
 static String     *destructor_action = 0;
@@ -115,6 +119,9 @@ public:
 	  Swig_mark_arg(i);
 	} else if (strcmp(argv[i],"-help") == 0) {
 	  fputs(usage,stderr);
+	} else if (strcmp (argv[i], "-ldflags") == 0) {
+	  printf("%s\n", SWIG_TCL_RUNTIME);
+	  SWIG_exit (EXIT_SUCCESS);
 	}
       }
     }
@@ -158,6 +165,11 @@ public:
     
     Swig_banner(f_runtime);
     
+    /* Include a Tcl configuration file */
+    if (NoInclude) {
+      Printf(f_runtime,"#define SWIG_NOINCLUDE\n");
+    }
+    
     /* Set the module name, namespace, and prefix */
     
     module = NewStringf("%(lower)s", Getattr(n,"name"));
@@ -338,7 +350,7 @@ public:
 	  }
 
 	  Putc('o',argstr);
-	  Printf(args,",NULL");
+	  Printf(args,",0");
 	  if (i >= num_required) {
 	    Printf(incode, "if (objc > %d) {\n", i+1);
 	  }
@@ -353,7 +365,7 @@ public:
 	    SwigType *lt = SwigType_ltype(pt);
 	    SwigType_remember(pt);
 	    if (Cmp(lt,"p.void") == 0) {
-	      Printf(args,",NULL");
+	      Printf(args,",0");
 	    } else {
 	      Printf(args,",SWIGTYPE%s", SwigType_manglestr(pt));
 	    }
@@ -614,7 +626,6 @@ public:
   virtual int constantWrapper(Node *n) {
     String *name      = Getattr(n,"name");
     String *iname     = Getattr(n,"sym:name");
-    String *nsname    = !nspace ? Copy(iname) : NewStringf("%s::%s",ns_name,iname);
     SwigType *type    = Getattr(n,"type");
     String   *value   = Getattr(n,"value");
     String *tm;
@@ -627,26 +638,21 @@ public:
       Printf(f_wrappers, "static %s = %s;\n", SwigType_str(type,wname), value);
       value = Char(wname);
     }
-    
     if ((tm = Swig_typemap_lookup_new("consttab",n,name,0))) {
       Replaceall(tm,"$source",value);
       Replaceall(tm,"$target",name);
       Replaceall(tm,"$value",value);
-      Replaceall(tm,"$nsname",nsname);
       Printf(const_tab,"%s,\n", tm);
     } else if ((tm = Swig_typemap_lookup_new("constcode", n, name, 0))) {
       Replaceall(tm,"$source", value);
       Replaceall(tm,"$target", name);
       Replaceall(tm,"$value",value);
-      Replaceall(tm,"$nsname",nsname);
       Printf(f_init, "%s\n", tm);
     } else {
-      Delete(nsname);
       Swig_warning(WARN_TYPEMAP_CONST_UNDEF,
 		   input_file, line_number, "Unsupported constant value.\n");
       return SWIG_NOWRAP;
     }
-    Delete(nsname);
     return SWIG_OK;
   }
 
@@ -749,7 +755,6 @@ public:
     /* Handle inheritance */
   
     String *base_class = NewString("");
-    String *base_class_names = NewString("");
 
     if( itcl ) {
       base_classes    = NewString("");
@@ -775,11 +780,10 @@ public:
 	//	Printv(f_wrappers,"extern swig_class _wrap_class_", bmangle, ";\n", NIL);
 	//	Printf(base_class,"&_wrap_class_%s",bmangle);
 	Printf(base_class,"0");
-	Printf(base_class_names,"\"%s *\",", SwigType_namestr(bname));
 	/* Put code to register base classes in init function */
 	
-	//Printf(f_init,"/* Register base : %s */\n", bmangle);
-	//Printf(f_init,"swig_%s_bases[%d] = (swig_class *) SWIG_TypeQuery(\"%s *\")->clientdata;\n",  mangled_classname, index, SwigType_namestr(bname));
+	Printf(f_init,"/* Register base : %s */\n", bmangle);
+	Printf(f_init,"swig_%s_bases[%d] = (swig_class *) SWIG_TypeQuery(\"%s *\")->clientdata;\n",  mangled_classname, index, SwigType_namestr(bname));
 	b = Next(b);
 	index++;
 	Putc(',',base_class);
@@ -889,17 +893,13 @@ public:
     };
 
     Printv(f_wrappers,"static swig_class *swig_",mangled_classname,"_bases[] = {", base_class,"0};\n", NIL);
-    Printv(f_wrappers,"static char *swig_",mangled_classname,"_base_names[] = {", base_class_names,"0};\n", NIL);
     Delete(base_class);
-    Delete(base_class_names);
 
     Printv(f_wrappers, "swig_class _wrap_class_", mangled_classname, " = { \"", class_name,
 	   "\", &SWIGTYPE", SwigType_manglestr(t), ",",NIL);
   
     if (have_constructor) {
-      Printf(f_wrappers,"%s", Swig_name_wrapper(Swig_name_construct(constructor_name)));
-      Delete(constructor_name);
-      constructor_name = 0;
+      Printf(f_wrappers,"%s", Swig_name_wrapper(Swig_name_construct(class_name)));
     } else {
       Printf(f_wrappers,"0");
     }
@@ -908,8 +908,7 @@ public:
     } else {
       Printf(f_wrappers,",0");
     }
-    Printv(f_wrappers, ", swig_", mangled_classname, "_methods, swig_", mangled_classname, "_attributes, swig_", mangled_classname,"_bases,",
-		         "swig_", mangled_classname, "_base_names };\n", NIL);
+    Printv(f_wrappers, ", swig_", mangled_classname, "_methods, swig_", mangled_classname, "_attributes, swig_", mangled_classname,"_bases };\n", NIL);
 
     if( !itcl ) {
       Printv(cmd_tab, tab4, "{ SWIG_prefix \"", class_name, "\", (swig_wrapper_func) SWIG_ObjectConstructor, &_wrap_class_", mangled_classname, "},\n", NIL);
@@ -1150,7 +1149,6 @@ public:
       }
     }
 
-    constructor_name = NewString(Getattr(n, "sym:name"));
     have_constructor = 1;
     return SWIG_OK;
   }
@@ -1222,12 +1220,10 @@ public:
  * swig_tcl()    - Instantiate module
  * ---------------------------------------------------------------------- */
 
-static Language * new_swig_tcl() {
+extern "C" Language *
+swig_tcl(void) {
   return new TCL8();
 }
-extern "C" Language * swig_tcl(void) {
-  return new_swig_tcl();
-}
 
 
 
