@@ -21,7 +21,7 @@
  * See the file LICENSE for information on usage and redistribution.	
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_typepass_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/typepass.cxx,v 1.14 2004/08/23 20:09:00 wsfulton Exp $";
+char cvsroot_typepass_cxx[] = "$Header: /cvsroot/SWIG/Source/Modules/typepass.cxx,v 1.8 2004/01/09 19:36:58 beazley Exp $";
 
 #include "swigmod.h"
 
@@ -34,8 +34,7 @@ struct normal_node {
 
 static normal_node *patch_list = 0;
 
-/* Singleton class - all non-static methods in this class are private */
-class TypePass : private Dispatcher {
+class TypePass : public Dispatcher {
   Node   *inclass;
   Node   *module;
   int    importmode;
@@ -43,13 +42,18 @@ class TypePass : private Dispatcher {
   Hash   *classhash;
   List   *normalize;
 
-  TypePass() {
-  }
-
   /* Normalize a type. Replaces type with fully qualified version */
 
   void normalize_type(SwigType *ty) {
     SwigType *qty;
+    /*qty = Swig_symbol_type_qualify(ty,0);*/
+    /*      if (SwigType_istemplate(ty)) {
+	    qty = Swig_symbol_type_qualify(ty,0);
+	    Clear(ty);
+	    Append(ty,qty);
+	    }
+    */
+
     if (CPlusPlus) {
 	Replaceall(ty,"struct ","");
 	Replaceall(ty,"union ","");
@@ -255,8 +259,8 @@ class TypePass : private Dispatcher {
       if ((Strcmp(nodeType(nn),"template") == 0) ||
 	  (Getattr(nn,"feature:ignore")) ||
 	  (Getattr(nn,"error")) ||
-	  // (checkAttribute(nn,"storage","friend")) ||
-	  ((Strcmp(nodeType(nn),"using") == 0) && !firstChild(nn))) {
+	  ((Strcmp(nodeType(nn),"using") == 0) && !firstChild(nn)) ||
+	  (checkAttribute(nn,"storage","friend"))) {
 	/* Remove from overloaded list */
 	Node *ps = Getattr(nn,"sym:previousSibling");
 	Node *ns = Getattr(nn,"sym:nextSibling");
@@ -314,6 +318,8 @@ class TypePass : private Dispatcher {
     }
   }
 
+public:
+
   /* ------------------------------------------------------------
    * top()
    * ------------------------------------------------------------ */ 
@@ -394,19 +400,13 @@ class TypePass : private Dispatcher {
 	fname = SwigType_typedef_resolve_all(name);
 	if (Strcmp(fname,name) != 0) {
 	  cn = Swig_symbol_clookup_local(fname,0);
-	  if ((!cn) || (Strcmp(nodeType(cn),"template") == 0)
-	      || (Getattr(cn,"feature:onlychildren") != 0)
-	      || (Getattr(n,"feature:onlychildren") != 0)) {
+	  if ((!cn) || (Strcmp(nodeType(cn),"template") == 0)) {
 	    Swig_symbol_cadd(fname,n);
 	    SwigType_typedef_class(fname);
 	    scopename = Copy(fname);
 	  } else {
-	    Swig_warning(WARN_TYPE_REDEFINED,Getfile(n),Getline(n),
-			 "Template '%s' was already wrapped,\n",
-			 SwigType_namestr(name));
-	    Swig_warning(WARN_TYPE_REDEFINED,Getfile(cn), Getline(cn),
-			 "previous wrap of '%s'.\n",
-			 SwigType_namestr(Getattr(cn,"name")));
+	    Swig_warning(WARN_TYPE_REDEFINED,Getfile(n),Getline(n),"Template '%s' was already wrapped as '%s' at %s:%d.\n",
+			 SwigType_namestr(name), SwigType_namestr(Getattr(cn,"name")), Getfile(cn), Getline(cn));
 	    scopename = 0;
 	  }
 	} else {
@@ -737,29 +737,20 @@ class TypePass : private Dispatcher {
 
   virtual int enumDeclaration(Node *n) {
     String *name = Getattr(n,"name");
-
+    String *uname = Getattr(n,"unnamed");
     if (name) {
-      // Correct the name to contain the fully qualified scopename
-      // Add a typedef to the type table so that we can use 'enum Name' as well as just 'Name'
-      if (nsname || inclass) {
-        String *nname = 0;
-        if (nsname && inclass) {
-          nname = NewStringf("%s::%s::%s", nsname, Getattr(inclass,"name"), name);
-        } else if (nsname) {
-          nname = NewStringf("%s::%s", nsname, name);
-        } else if (inclass) {
-          nname = NewStringf("%s::%s", Getattr(inclass,"name"), name);
-        }
-        Setattr(n,"name",nname);
-        SwigType *t = NewStringf("enum %s", nname);
-        SwigType_typedef(t,name);
-        Delete(nname);
+      SwigType *t;
+      if (uname) {
+	t = NewStringf("enum %s", uname);
+	if (checkAttribute(n,"storage","typedef")) {
+	  SwigType_typedef(t,name);
+	}
       } else {
-        SwigType *t = NewStringf("enum %s", name);
-        SwigType_typedef(t,name);
+	t = NewStringf("enum %s", name);
+	SwigType_typedef(t,name);
       }
+      Delete(t);
     }
-
     emit_children(n);
     return SWIG_OK;
   }
@@ -782,30 +773,9 @@ class TypePass : private Dispatcher {
       Setattr(n,"value",new_value);
       Delete(new_value);
     }
-
-    // Make up an enumvalue if one was not specified in the parsed code
-    if (Getattr(n,"_last") && !Getattr(n,"enumvalue")) { // Only the first enum item has _last set
-      Setattr(n,"enumvalueex", "0");
-    }
-    Node *next = nextSibling(n);
-    if (next && !Getattr(next,"enumvalue")) {
-      Setattr(next,"enumvalueex", NewStringf("%s + 1", Getattr(n,"sym:name")));
-    }
-
     return SWIG_OK;
   }
 
-  /* ------------------------------------------------------------
-   * enumforwardDeclaration()
-   * ------------------------------------------------------------ */ 
-
-  virtual int enumforwardDeclaration(Node *n) {
-
-    // Use enumDeclaration() to do all the hard work.
-    // Note that no children can be emitted in a forward declaration as there aren't any.
-    return enumDeclaration(n);
-  }
-
   /* ------------------------------------------------------------
    * usingDeclaration()
    * ------------------------------------------------------------ */
@@ -1001,17 +971,13 @@ class TypePass : private Dispatcher {
     }
     return SWIG_OK;
   }
-
-public:
-  static void pass(Node *n) {
-    TypePass t;
-    t.top(n);
-  }
 };
 
 void Swig_process_types(Node *n) {
   if (!n) return;
-  TypePass::pass(n);
+  TypePass *t = new TypePass;
+  t->top(n);
+  delete t;
 }
 
 
