@@ -8,13 +8,12 @@
  * Copyright (C) 1999-2000.  The University of Chicago
  * See the file LICENSE for information on usage and redistribution.
  *
- * $Header: /cvs/projects/SWIG/Source/Preprocessor/preprocessor.h,v 1.5.4.8 2002/08/26 19:05:38 beazley Exp $
+ * $Header: /cvsroot/SWIG/Source/Preprocessor/preprocessor.h,v 1.8 2003/12/28 21:45:36 cheetah Exp $
  * ----------------------------------------------------------------------------- */
 
-#ifndef _PREPROCESSOR_H
-#define _PREPROCESSOR_H
+#ifndef PREPROCESSOR_H_
+#define PREPROCESSOR_H_
 
-#include "swig.h"
 #include "swigwarn.h"
 
 #ifdef __cplusplus
@@ -30,6 +29,7 @@ extern void    Preprocessor_include_all(int);
 extern void    Preprocessor_import_all(int);
 extern void    Preprocessor_ignore_missing(int);
 extern List   *Preprocessor_depend(void);
+extern void    Preprocessor_expr_init(void);
 
 #ifdef __cplusplus
 }
