@@ -9,17 +9,21 @@
  * Copyright (C) 1999-2000.  The University of Chicago
  * See the file LICENSE for information on usage and redistribution.	
  *
- * $Header: /cvs/projects/SWIG/Source/Swig/swig.h,v 1.69 2003/03/08 03:25:09 arty Exp $
+ * $Header: /cvsroot/SWIG/Source/Swig/swig.h,v 1.78 2004/01/10 00:04:17 cheetah Exp $
  * ----------------------------------------------------------------------------- */
 
-#ifndef _SWIGCORE_H
-#define _SWIGCORE_H
+#ifndef SWIGCORE_H_
+#define SWIGCORE_H_
 
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
 #include <assert.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 #include "doh.h"
 
 /* Status codes */
@@ -84,6 +88,8 @@ typedef  DOH     SwigType;
 /* --- File interface --- */
 
 extern void     Swig_add_directory(const String_or_char *dirname);
+extern void     Swig_push_directory(const String_or_char *dirname);
+extern void     Swig_pop_directory();
 extern String  *Swig_last_file();
 extern List    *Swig_search_path();
 extern FILE    *Swig_open(const String_or_char *name);
@@ -101,8 +107,10 @@ extern char    *Swig_file_basename(const String_or_char *filename);
 extern char    *Swig_file_filename(const String_or_char *filename);
 extern char    *Swig_file_dirname(const String_or_char *filename);
 
-#ifdef MACSWIG
+#if defined(MACSWIG)
 #  define SWIG_FILE_DELIMETER  ":"
+#elif defined(_WIN32)
+#  define  SWIG_FILE_DELIMETER "\\"
 #else
 #  define  SWIG_FILE_DELIMETER "/"
 #endif
@@ -190,13 +198,16 @@ extern void         SwigScanner_idstart(SwigScanner *, char *idchar);
 /* --- Functions for manipulating the string-based type encoding --- */
 
 extern SwigType   *NewSwigType(int typecode);
+extern void        SwigType_del_element(SwigType *t);
 extern void        SwigType_add_pointer(SwigType *t);
 extern void        SwigType_add_memberpointer(SwigType *t, String_or_char *qual);
 extern void        SwigType_del_pointer(SwigType *t);
 extern void        SwigType_add_array(SwigType *t, String_or_char *size);
 extern SwigType   *SwigType_pop_arrays(SwigType *t);
 extern void        SwigType_add_reference(SwigType *t);
+extern void        SwigType_del_reference(SwigType *t);
 extern void        SwigType_add_qualifier(SwigType *t, String_or_char *qual);
+extern void        SwigType_del_qualifier(SwigType *t);
 extern void        SwigType_add_function(SwigType *t, ParmList *parms);
 extern void        SwigType_add_template(SwigType *t, ParmList *parms);
 extern SwigType   *SwigType_pop_function(SwigType *t);
@@ -251,8 +262,6 @@ extern int         SwigType_issubtype(SwigType *subtype, SwigType *basetype);
 extern void        SwigType_scope_alias(String *aliasname, Typetab *t);
 extern void        SwigType_using_scope(Typetab *t);
 extern void        SwigType_new_scope(String_or_char *name);
-extern void        SwigType_reset_scopes();
-extern void        SwigType_set_scope_name(String_or_char *name);
 extern void        SwigType_inherit_scope(Typetab *scope);
 extern Typetab    *SwigType_pop_scope();
 extern Typetab    *SwigType_set_scope(Typetab *h);
@@ -342,7 +351,6 @@ extern int  Swig_save(const char *ns, Node *node,...);
 extern void Swig_restore(Node *node);
 
 /* Debugging of parse trees */
-extern void Swig_debug_emit(int);
 extern void Swig_print_tags(File *obj, Node *root);
 extern void Swig_print_tree(Node *obj);
 extern void Swig_print_node(Node *obj);
@@ -407,6 +415,8 @@ extern void       Swig_warn(const char *filename, int line, const char *msg);
 
 #define WARNING(msg) Swig_warn(__FILE__,__LINE__,msg)
 
+typedef enum { EMF_STANDARD, EMF_MICROSOFT } ErrorMessageFormat;
+
 extern void       Swig_warning(int num, const String_or_char *filename, int line, const char *fmt, ...);
 extern void       Swig_error(const String_or_char *filename, int line, const char *fmt, ...);
 extern int        Swig_error_count(void);
@@ -414,6 +424,7 @@ extern void       Swig_error_silent(int s);
 extern void       Swig_warnfilter(const String_or_char *wlist, int val);
 extern void       Swig_warnall(void);
 extern int        Swig_warn_count(void);
+extern void       Swig_error_msg_format(ErrorMessageFormat format);
 
 /* --- C Wrappers --- */
 extern String    *Swig_cparm_name(Parm *p, int i);
@@ -438,6 +449,7 @@ extern String    *Swig_cmemberget_call(String_or_char *name, SwigType *t, String
 extern int        Swig_MethodToFunction(Node *n, String *classname, int flags);
 extern int        Swig_ConstructorToFunction(Node *n, String *classname, 
 					     String *none_comparison,
+					     String *director_ctor,
 					     int cplus, int flags);
 extern int        Swig_DestructorToFunction(Node *n, String *classname, int cplus, int flags);
 extern int        Swig_MembersetToFunction(Node *n, String *classname, int flags);
@@ -471,8 +483,7 @@ extern String *Swig_typemap_lookup(const String_or_char *op, SwigType *type, Str
 				  String_or_char *source, String_or_char *target, Wrapper *f);
 
 extern String *Swig_typemap_lookup_new(const String_or_char *op, Node *n, const String_or_char *lname, Wrapper *f);
-
-extern String *Swig_typemap_lookup_multi(const String_or_char *op, ParmList *parms, String_or_char *source, Wrapper *f, int *nmatch);
+extern void Swig_typemap_attach_kwargs(Hash *tm, const String_or_char *op, Parm *p);
 extern void   Swig_typemap_new_scope();
 extern Hash  *Swig_typemap_pop_scope();
 
@@ -482,13 +493,13 @@ extern void   Swig_typemap_attach_parms(const String_or_char *op, ParmList *parm
 
 extern  void  Swig_fragment_register(String *name, String *section, String *code);
 extern  void  Swig_fragment_emit(String *name);
-  
+
+/* hacks defined in C++ ! */
+extern  int   Swig_need_protected();
+
+#ifdef __cplusplus
+}
 #endif
 
-
-
-
-
-
-
+#endif
 
