@@ -12,7 +12,7 @@
  * See the file LICENSE for information on usage and redistribution.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_typesys_c[] = "$Header: /cvs/projects/SWIG/Source/Swig/typesys.c,v 1.4 2003/03/27 23:17:27 beazley Exp $";
+char cvsroot_typesys_c[] = "$Header: /cvsroot/SWIG/Source/Swig/typesys.c,v 1.10 2003/11/26 20:56:44 beazley Exp $";
 
 #include "swig.h"
 
@@ -366,27 +366,26 @@ SwigType_attach_symtab(Symtab *sym) {
  * ----------------------------------------------------------------------------- */
 
 void SwigType_print_scope(Typetab *t) {
-  String *key;
   Hash   *ttab;
-  String *tkey;
+  Iterator  i,j;
 
-  for (tkey = Firstkey(scopes); tkey; tkey = Nextkey(scopes)) {
-    t = Getattr(scopes,tkey);
-    ttab = Getattr(t,"typetab");
+  for (i = First(scopes); i.key; i = Next(i)) {
+    t = i.item;
+    ttab = Getattr(i.item,"typetab");
     
-    Printf(stdout,"Type scope '%s' (%x)\n", tkey, t);
+    Printf(stdout,"Type scope '%s' (%x)\n", i.key, i.item);
     {
-      List *inherit = Getattr(t,"inherit");
+      List *inherit = Getattr(i.item,"inherit");
       if (inherit) {
-	Typetab *it;
-	for (it = Firstitem(inherit); it; it = Nextitem(inherit)) {
-	  Printf(stdout,"    Inherits from '%s' (%x)\n", Getattr(it,"qname"), it);
+	Iterator j;
+	for (j = First(inherit); j.item; j = Next(j)) {
+	  Printf(stdout,"    Inherits from '%s' (%x)\n", Getattr(j.item,"qname"), j.item);
 	}
       }
     }
     Printf(stdout,"-------------------------------------------------------------\n");
-    for (key = Firstkey(ttab); key; key = Nextkey(ttab)) {
-      Printf(stdout,"%40s -> %s\n", key, Getattr(ttab,key));
+    for (j = First(ttab); j.key; j = Next(j)) {
+      Printf(stdout,"%40s -> %s\n", j.key, j.item);
     }
   }
 }
@@ -507,7 +506,6 @@ SwigType *SwigType_typedef_resolve(SwigType *t) {
   String *namebase = 0;
   String *nameprefix = 0;
   int     newtype = 0;
-  static   String *noscope = 0;
 
   /*
   if (!noscope) {
@@ -785,12 +783,13 @@ SwigType *SwigType_typedef_qualified(SwigType *t)
   String *result;
   int     i,len;
 
- if (!typedef_qualified_cache) typedef_qualified_cache = NewHash();
+  if (!typedef_qualified_cache) typedef_qualified_cache = NewHash();
   result = Getattr(typedef_qualified_cache,t);
   if (result) {
     String *rc = Copy(result);
     return rc;
   }
+
   result = NewString("");
   elements = SwigType_split(t);
   len = Len(elements);
@@ -817,7 +816,6 @@ SwigType *SwigType_typedef_qualified(SwigType *t)
 	    }
 	  }
 	} else {
-
 	  if (Swig_scopename_check(e)) {
 	    String *tqname;
 	    String *qlast;
@@ -832,7 +830,27 @@ SwigType *SwigType_typedef_qualified(SwigType *t)
 	      Delete(tqname);
 	    }
 	    /* Automatic template instantiation might go here??? */
-
+	  } else {
+	    /* It's a bare name.  It's entirely possible, that the
+               name is part of a namespace. We'll check this by unrolling
+               out of the current scope */
+	    
+	    Typetab *cs = current_scope;
+	    while (cs) {
+	      String *qs = SwigType_scope_name(cs);
+	      if (Len(qs)) {
+		Append(qs,"::");
+	      }
+	      Append(qs,e);
+	      if (Getattr(scopes,qs)) {
+		Clear(e);
+		Append(e,qs);
+		Delete(qs);
+		break;
+	      }
+	      Delete(qs);
+	      cs = Getattr(cs,"parent");
+	    }
 	  }
 	}
 	if (isenum) e = isenum;
@@ -841,14 +859,15 @@ SwigType *SwigType_typedef_qualified(SwigType *t)
 	/* Template.  We need to qualify template parameters as well as the template itself */
 	String *tprefix, *qprefix;
 	String *tsuffix;
+	Iterator pi;
 	Parm   *p;
 	List *parms = SwigType_parmlist(e);
 	tprefix = SwigType_templateprefix(e);
 	tsuffix = SwigType_templatesuffix(e);
 	qprefix = SwigType_typedef_qualified(tprefix);
-	Printf(qprefix,"<(");
-	p = Firstitem(parms);
-	while (p) {
+	Printv(qprefix,"<(",NIL);
+	pi = First(parms);
+	while ((p = pi.item)) {
 	  String *qt = SwigType_typedef_qualified(p);
 	  if ((Strcmp(qt,p) == 0)) { /*  && (!Swig_scopename_check(qt))) { */
 	    /* No change in value.  It is entirely possible that the parameter is an integer value.
@@ -892,8 +911,8 @@ SwigType *SwigType_typedef_qualified(SwigType *t)
 	    Append(qprefix,qt);
 	  }
 	  Delete(qt);
-	  p= Nextitem(parms);
-	  if (p) {
+	  pi= Next(pi);
+	  if (pi.item) {
 	    Append(qprefix,",");
 	  }
 	}
@@ -914,14 +933,14 @@ SwigType *SwigType_typedef_qualified(SwigType *t)
     } else if (SwigType_isfunction(e)) {
       List *parms = SwigType_parmlist(e);
       String *s = NewString("f(");
-      String *p;
-      p = Firstitem(parms);
-      while (p) {
-	String *pq = SwigType_typedef_qualified(p);
+      Iterator pi;
+      pi = First(parms);
+      while (pi.item) {
+	String *pq = SwigType_typedef_qualified(pi.item);
 	Append(s,pq);
 	Delete(pq);
-	p = Nextitem(parms);
-	if (p) {
+	pi = Next(pi);
+	if (pi.item) {
 	  Append(s,",");
 	}
       }
@@ -1070,9 +1089,11 @@ SwigType_isclass(SwigType *t) {
       isclass = 1;
     }
     /* Hmmm. Not a class.  If a template, it might be uninstantiated */
-    if (SwigType_istemplate(qtys)) {
+    if (!isclass && SwigType_istemplate(qtys)) {
       String *tp = SwigType_templateprefix(qtys);
-      isclass = SwigType_isclass(tp);
+      if (Strcmp(tp,t) != 0) {
+	isclass = SwigType_isclass(tp);
+      }
       Delete(tp);
     }
   }
@@ -1227,10 +1248,11 @@ void SwigType_remember_clientdata(SwigType *t, const String_or_char *clientdata)
     (*r_tracefunc)(t,mt, (String *) clientdata);
   }
 
-  if (SwigType_istypedef(t))
+  if (SwigType_istypedef(t)) {
     lt = Copy(t);
-  else
+  } else {
     lt = SwigType_ltype(t);
+  }
   Setattr(r_ltype, mt, lt);
   fr = SwigType_typedef_resolve_all(t);     /* Create fully resolved type */
   qr = SwigType_typedef_qualified(fr);
@@ -1276,6 +1298,17 @@ void SwigType_remember_clientdata(SwigType *t, const String_or_char *clientdata)
       Setattr(r_clientdata, fr, NewString(clientdata));
     }
   }
+
+  /* If the remembered type is a reference, we also remember the pointer version.
+     This is to prevent odd problems with mixing pointers and references--especially
+     when different functions are using different typenames (via typedef). */
+
+  if (SwigType_isreference(t)) {
+    SwigType *tt = Copy(t);
+    SwigType_del_reference(tt);
+    SwigType_add_pointer(tt);
+    SwigType_remember_clientdata(tt,clientdata);
+  }
 }
 
 void
@@ -1319,26 +1352,26 @@ List *SwigType_equivalent_mangle(String *ms, Hash *checked, Hash *found) {
   Setattr(ch, ms, "1");
   mh = Getattr(r_mangled,ms);
   if (mh) {
-    String *key;
-    key = Firstkey(mh);
-    while (key) {
+    Iterator ki;
+    ki = First(mh);
+    while (ki.key) {
       Hash *rh;
-      if (Getattr(ch,key)) {
-	key = Nextkey(mh);
+      if (Getattr(ch,ki.key)) {
+	ki = Next(ki);
 	continue;
       }
-      Setattr(ch,key,"1");
-      rh = Getattr(r_resolved,key);
+      Setattr(ch,ki.key,"1");
+      rh = Getattr(r_resolved,ki.key);
       if (rh) {
-	String *rkey;
-	rkey = Firstkey(rh);
-	while (rkey) {
-	  Setattr(h,rkey,"1");
-	  SwigType_equivalent_mangle(rkey,ch,h);
-	  rkey = Nextkey(rh);
+	Iterator rk;
+	rk = First(rh);
+	while (rk.key) {
+	  Setattr(h,rk.key,"1");
+	  SwigType_equivalent_mangle(rk.key,ch,h);
+	  rk = Next(rk);
 	}
       }
-      key = Nextkey(mh);
+      ki = Next(ki);
     }
   }
  check_exit:
@@ -1373,24 +1406,24 @@ String *SwigType_clientdata_collect(String *ms, Hash *checked) {
   Setattr(ch, ms, "1");
   mh = Getattr(r_mangled,ms);
   if (mh) {
-    String *key;
-    key = Firstkey(mh);
-    while (key) {
+    Iterator ki;
+    ki = First(mh);
+    while (ki.key) {
       Hash *rh;
-      Setattr(ch,key,"1");
-      clientdata = Getattr(r_clientdata,key);
+      Setattr(ch,ki.key,"1");
+      clientdata = Getattr(r_clientdata,ki.key);
       if (clientdata) goto check_exit;
-      rh = Getattr(r_resolved,key);
+      rh = Getattr(r_resolved,ki.key);
       if (rh) {
-	String *rkey;
-	rkey = Firstkey(rh);
-	while (rkey) {
-	  clientdata = SwigType_clientdata_collect(rkey,ch);
+	Iterator rk;
+	rk = First(rh);
+	while (rk.key) {
+	  clientdata = SwigType_clientdata_collect(rk.key,ch);
 	  if (clientdata) goto check_exit;
-	  rkey = Nextkey(rh);
+	  rk = Next(rk);
 	}
       }
-      key = Nextkey(mh);
+      ki = Next(ki);
     }
   }
  check_exit:
@@ -1484,48 +1517,49 @@ SwigType_issubtype(SwigType *t1, SwigType *t2) {
  * ----------------------------------------------------------------------------- */
 
 void SwigType_inherit_equiv(File *out) {
-  String *rkey, *bkey, *ckey;
+  String *ckey;
   String *prefix, *base;
   Hash   *sub;
   Hash   *rh;
   List   *rlist;
+  Iterator rk, bk, ck;
 
   if (!conversions) conversions = NewHash();
   if (!subclass) subclass = NewHash();
 
-  rkey = Firstkey(r_resolved);
-  while (rkey) {
+  rk = First(r_resolved);
+  while (rk.key) {
     /* rkey is a fully qualified type.  We strip all of the type constructors off of it just to get the base */
-    base = SwigType_base(rkey);
+    base = SwigType_base(rk.key);
     /* Check to see whether the base is recorded in the subclass table */
     sub = Getattr(subclass,base);
     Delete(base);
     if (!sub) {
-      rkey = Nextkey(r_resolved);
+      rk = Next(rk);
       continue;
     }
 
     /* This type has subclasses.  We now need to walk through these subtypes and generate pointer converion functions */
 
-    rh = Getattr(r_resolved, rkey);
+    rh = Getattr(r_resolved, rk.key);
     rlist = NewList();
-    for (ckey = Firstkey(rh); ckey; ckey = Nextkey(rh)) {
-      Append(rlist,ckey);
+    for (ck = First(rh); ck.key; ck = Next(ck)) {
+      Append(rlist,ck.key);
     }
-    /*    Printf(stdout,"rkey = '%s'\n", rkey);
+    /*    Printf(stdout,"rk.key = '%s'\n", rk.key);
 	  Printf(stdout,"rh = %x '%s'\n", rh,rh); */
 
-    bkey = Firstkey(sub);
-    while (bkey) {
-      prefix= SwigType_prefix(rkey);
-      Append(prefix,bkey);
+    bk = First(sub);
+    while (bk.key) {
+      prefix= SwigType_prefix(rk.key);
+      Append(prefix,bk.key);
       /*      Printf(stdout,"set %x = '%s' : '%s'\n", rh, SwigType_manglestr(prefix),prefix); */
       Setattr(rh,SwigType_manglestr(prefix),prefix);
-      ckey = NewStringf("%s+%s",SwigType_manglestr(prefix), SwigType_manglestr(rkey));
+      ckey = NewStringf("%s+%s",SwigType_manglestr(prefix), SwigType_manglestr(rk.key));
       if (!Getattr(conversions,ckey)) {
-	String *convname = NewStringf("%sTo%s", SwigType_manglestr(prefix), SwigType_manglestr(rkey));
+	String *convname = NewStringf("%sTo%s", SwigType_manglestr(prefix), SwigType_manglestr(rk.key));
 	Printf(out,"static void *%s(void *x) {\n", convname);
-	Printf(out,"    return (void *)((%s) %s ((%s) x));\n", SwigType_lstr(rkey,0), Getattr(sub,bkey), SwigType_lstr(prefix,0));
+	Printf(out,"    return (void *)((%s) %s ((%s) x));\n", SwigType_lstr(rk.key,0), Getattr(sub,bk.key), SwigType_lstr(prefix,0));
 	Printf(out,"}\n");
 	Setattr(conversions,ckey,convname);
 	Delete(ckey);	
@@ -1534,39 +1568,40 @@ void SwigType_inherit_equiv(File *out) {
 	{
 	  Hash *r = Getattr(r_resolved, prefix);
 	  if (r) {
-	    String *rrkey = Firstkey(r);
-	    while (rrkey) {
-	      String *rlkey;
+	    Iterator rrk;
+	    rrk=First(r);
+	    while (rrk.key) {
+	      Iterator rlk;
 	      String *rkeymangle;
 
 	      /* Make sure this name equivalence is not due to inheritance */
-	      if (Cmp(prefix, Getattr(r,rrkey)) == 0) {
-		rkeymangle = SwigType_manglestr(rkey);
-		ckey = NewStringf("%s+%s", rrkey, rkeymangle);
+	      if (Cmp(prefix, Getattr(r,rrk.key)) == 0) {
+		rkeymangle = SwigType_manglestr(rk.key);
+		ckey = NewStringf("%s+%s", rrk.key, rkeymangle);
 		if (!Getattr(conversions, ckey)) {
 		  Setattr(conversions, ckey, convname);
 		}
 		Delete(ckey);
-		for (rlkey = Firstitem(rlist); rlkey; rlkey = Nextitem(rlist)) {
-		  ckey = NewStringf("%s+%s", rrkey, rlkey);
+		for (rlk = First(rlist); rlk.item; rlk = Next(rlk)) {
+		  ckey = NewStringf("%s+%s", rrk.key, rlk.item);
 		  Setattr(conversions, ckey, convname);
 		  Delete(ckey);
 		}
 		Delete(rkeymangle);
 		/* This is needed to pick up other alternative names for the same type.
                    Needed to make templates work */
-	        Setattr(rh,rrkey,Getattr(r,rrkey));   
+	        Setattr(rh,rrk.key,rrk.item);   
 	      }
-	      rrkey = Nextkey(r);
+	      rrk = Next(rrk);
 	    }
 	  }
 	}
 	Delete(convname);
       }
       Delete(prefix);
-      bkey = Nextkey(sub);
+      bk = Next(bk);
     }
-    rkey = Nextkey(r_resolved);
+    rk = Next(rk);
   }
 }
 
@@ -1578,7 +1613,7 @@ void SwigType_inherit_equiv(File *out) {
 
 void
 SwigType_emit_type_table(File *f_forward, File *f_table) {
-  DOH *key;
+  Iterator ki;
   String *types, *table;
   int i = 0;
 
@@ -1612,36 +1647,37 @@ SwigType_emit_type_table(File *f_forward, File *f_table) {
   table = NewString("");
   types = NewString("");
   Printf(table,"static swig_type_info *swig_types_initial[] = {\n");
-  key = Firstkey(r_mangled);
+
+  ki = First(r_mangled);
   Printf(f_forward,"\n/* -------- TYPES TABLE (BEGIN) -------- */\n\n");
-  while (key) {
+  while (ki.key) {
     List *el;
-    String *en;
+    Iterator ei;
     String *cd;
 
-    Printf(f_forward,"#define  SWIGTYPE%s swig_types[%d] \n", key, i);
-    Printv(types,"static swig_type_info _swigt_", key, "[] = {", NIL);
+    Printf(f_forward,"#define  SWIGTYPE%s swig_types[%d] \n", ki.key, i);
+    Printv(types,"static swig_type_info _swigt_", ki.key, "[] = {", NIL);
     
-    cd = SwigType_clientdata_collect(key,0);
+    cd = SwigType_clientdata_collect(ki.key,0);
     if (!cd) cd = "0";
-    Printv(types,"{\"", key, "\", 0, \"", SwigType_str(Getattr(r_ltype,key),0),"\", ", cd, "},", NIL);
-    el = SwigType_equivalent_mangle(key,0,0);
-    for (en = Firstitem(el); en; en = Nextitem(el)) {
+    Printv(types,"{\"", ki.key, "\", 0, \"", SwigType_str(Getattr(r_ltype,ki.key),0),"\", ", cd, "},", NIL);
+    el = SwigType_equivalent_mangle(ki.key,0,0);
+    for (ei = First(el); ei.item; ei = Next(ei)) {
       String *ckey;
       String *conv;
-      ckey = NewStringf("%s+%s", en, key);
+      ckey = NewStringf("%s+%s", ei.item, ki.key);
       conv = Getattr(conversions,ckey);
       if (conv) {
-	Printf(types,"{\"%s\", %s},", en, conv);
+	Printf(types,"{\"%s\", %s},", ei.item, conv);
       } else {
-	Printf(types,"{\"%s\"},", en);
+	Printf(types,"{\"%s\"},", ei.item);
       }
       Delete(ckey);
     }
     Delete(el);
     Printf(types,"{0}};\n");
-    Printv(table, "_swigt_", key, ", \n", NIL);
-    key = Nextkey(r_mangled);
+    Printv(table, "_swigt_", ki.key, ", \n", NIL);
+    ki = Next(ki);
     i++;
   }
 
